/**********************************************************************
 *
 * Copyright(C) 2011, Embedded Artists AB
 * All rights reserved.
 *
 ***********************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 **********************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/

#ifdef __USE_CMSIS
#include "LPC11Uxx.h"
#endif

#include <stdio.h>

#include "type.h"
#include "general.h"
#include "application.h"
#include "systick.h"
#include "gpio.h"
#include "adc.h"
#include "ls013b4dn04.h"
#include "s25fl004.h"
#include "i2c.h"
#include "pcf8523.h"
#include "pcf8885.h"
#include "adxl345.h"
#include "lm75b.h"
#include "pwrapi.h"
#include "pmu.h"

/******************************************************************************
 * Local variables
 *****************************************************************************/

static int16_t midScale[3];
static int16_t buffer[3];

/******************************************************************************
 * Local functions
 *****************************************************************************/


/******************************************************************************
 * Main function
 *****************************************************************************/

int main(void)
{
  LPC_SYSCON->PDRUNCFG &= ~(WDT_OSC_PD | ADC_PD);
  LPC_SYSCON->PDRUNCFG |= (SYS_OSC_PD);

  GPIOInit();

  prepareIoForSleep(MAKE_SPI0_INACTIVE, MAKE_CLKOUT_INACTIVE);

  //Set LED pin (PIO0_7) to output and turn LED off
  EA_LED_INIT;
  EA_LED_OFF;

  //Turn on-chip USB peripheral block off
  LPC_SYSCON->PDRUNCFG |= (1 << 10);	//Power-down USB PHY
  LPC_SYSCON->PDRUNCFG |= (1 <<  8);	//Power-down USB PLL
  //  LPC_SYSCON->PDRUNCFG |= (IRC_OUT_PD | IRC_PD);  //Power-down IRC

  LPC_SYSCON->PDRUNCFG &= ~(WDT_OSC_PD);

  LPC_SYSCON->STARTERP0 = (0x1<<0);			//Flex IRQ0 (Accel irq) can startup mcu
  LPC_SYSCON->STARTERP1 = (1<<12) | (1<<13);	//WDT and BOD can startup mcu

  //Turn LCD on
  PWR_LCD_INIT;
  PWR_LCD_ON;

  //Turn SPI-FLASH off
  PWR_FLASH_INIT;
  PWR_FLASH_OFF;

  //Init and enable I2C pull-up resistors
  PWR_I2C_INIT;
  PWR_I2C_ON;

  if ( I2CInit( (uint32_t)I2CMASTER ) == FALSE )  /* initialize I2c */
  {
    while ( 1 );        /* Fatal error */
  }

  //put capacitive sensor to sleep
  pcf8885_init();
  PCF8885_SLEEP_PIN_HIGH;

  //Init accelerometer
  adxl345_init();
  adxl345_setPowerControl(0x00);
  adxl345_setDataFormatControl(0x2B);		//Full resolution, +/-16g, 4mg/LSB.
  adxl345_setDataRate(ADXL345_3200HZ);		//3.2kHz data rate.
  adxl345_setPowerControl(MeasurementMode);	//Measurement mode.
  adxl345_setTapThreshold(0x38);
  adxl345_setTapDuration(20*625);
  adxl345_setTapLatency(0x50);
  adxl345_setWindowTime(0xff);				//255ms to second tap window
  adxl345_setTapAxisControl(0x01);			//tap detection in z-axis
  adxl345_setInterruptMappingControl(0x9F); //Send the Tap and Double Tap Interrupts to INT1 pin
  adxl345_setInterruptEnableControl(0x60);  //Enable the Single and Double Taps.
  adxl345_getInterruptSource();             //Clear the interrupts from the INT_SOURCE register.
  adxl345_init_irq();

  //Init pin that controls voltage to trimming potentiometer
  PWR_ADC_INIT;
  PWR_ADC_OFF;

  //Init the temperature sensor in power down mode
  lm75b_config(LM75B_CFG_SHUTDOWN);

  //Init the RTC
  //	pcf8523_init();

  lcd_init();
  lcd_rect(1, 1, 94, 94, LCD_COLOR_BLACK);
  lcd_putString(6,  4, "Accel demo", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
  lcd_line     (5, 12, 64, 12, LCD_COLOR_BLACK, FALSE);
  lcd_line     (5, 14, 64, 14, LCD_COLOR_BLACK, FALSE);

  lcd_rect(20, 20, 74, 74, LCD_COLOR_BLACK);
  buffer[0] = 0;
  buffer[1] = 0;
  buffer[2] = 0;
  midScale[0] = 0;
  midScale[1] = 0;
  midScale[2] = 0;

  initSysTickCounter(10);	//turn on 10Hz tick

  //Enter forever loop...
  while(1)
  {
    static uint8_t countDown = 0;

    prepareIoForSleep(KEEP_SPI0_ACTIVE, MAKE_CLKOUT_INACTIVE);
    LPC_IOCON->TDO_PIO0_13	= (1<<0) | (2<<3) | (1<<7);  //GPIO (not TDO), pullup on accel irq pin (PIO0_13)
    PMU_Sleep(MCU_SLEEP, (BOD_PD | WDT_OSC_PD)); //Power down BOD and WDT oscillator

    //Check if rtc interrupt (is the only one possible, but check anyways)
    if (TRUE == accelFlag)
    {
      accelFlag = FALSE;

      if (accelIrqSrc & (1<<5))
        lcd_putString(6,  75, "Double tap!", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, TRUE);
      else if (accelIrqSrc & (1<<6))
        lcd_putString(6,  85, "Single tap!", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, TRUE);
      countDown = 10;
    }

    //systick/wdt wakeup
    if (TRUE == sysTickFlag)
    {
      sysTickFlag = FALSE;

      if (countDown > 0)
      {
        countDown--;
        if (countDown == 0)
        {
          lcd_putString(6,  75, "           ", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
          lcd_putString(6,  85, "           ", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
        }
      }

      //get xy-accelerometer values
      lcd_line(47, 47, 47+(buffer[0]/6), 47-(buffer[1]/6), LCD_COLOR_WHITE, FALSE);
      adxl345_getOutput((uint16_t*)&buffer[0]);
      buffer[0] -= midScale[0];
      buffer[1] -= midScale[1];
      buffer[2] -= midScale[2];

      lcd_line(47, 47, 47+(buffer[0]/6), 47-(buffer[1]/6), LCD_COLOR_BLACK, FALSE);

      //update LCD
      lcd_update_vcom();
      lcd_updateScreen(0, 95);
    }

    //check if wakeup-key is pressed = calibrate accelerometer
    if ((LPC_GPIO->PIN[0] & (1<<16)) == 0)
    {
      int16_t buffer[3];
      int32_t resBuffer[3];
      uint8_t i;

      resBuffer[0] = 0;
      resBuffer[1] = 0;
      resBuffer[2] = 0;
      //get 30 samples
      for(i=0; i<32; i++)
      {
        adxl345_getOutput((uint16_t*)&buffer[0]);
        resBuffer[0] += buffer[0];
        resBuffer[1] += buffer[1];
        resBuffer[2] += buffer[2];
      }
      midScale[0] = resBuffer[0] / 32;
      midScale[1] = resBuffer[1] / 32;
      midScale[2] = resBuffer[2] / 32;
    }
  }

  return 0 ;
}


