/**********************************************************************
 *
 * Copyright(C) 2011, Embedded Artists AB
 * All rights reserved.
 *
 ***********************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 **********************************************************************/


/******************************************************************************
 * Includes
 *****************************************************************************/
#include "type.h"
#include "general.h"
#include "application.h"
#include "systick.h"
#include "ls013b4dn04.h"
#include "pcf8523.h"
#include "pcf8885.h"
#include "adxl345.h"
#include "pmu.h"
#include "gpio.h"
#include "lpc_winfreesystem14x16.h"
#include "pwrapi.h"

/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/

#define ALWAYS_READ_RTC

#define CURSOR_YEAR_1  (0)
#define CURSOR_YEAR_2  (1)
#define CURSOR_MONTH_1 (2)
#define CURSOR_MONTH_2 (3)
#define CURSOR_DAY_1   (4)
#define CURSOR_DAY_2   (5)
#define CURSOR_HOUR_1  (6)
#define CURSOR_HOUR_2  (7)
#define CURSOR_MIN_1   (8)
#define CURSOR_MIN_2   (9)
#define CURSOR_WEEKDAY (10)
#define CURSOR_UPD_SEC (11)
#define CURSOR_OK      (12)
#define CURSOR_CANCEL  (13)
#define CURSOR_NUM     (14)

#define DATE_Y    (18)
#define TIME_Y    (30)
#define DAY_Y     (42)
#define UPD_SEC_Y (54)
#define BUTTON_Y  (76)

#define CLOCK_SET_DELAY (300)


/******************************************************************************
 * Global variables
 *****************************************************************************/

/******************************************************************************
 * Local variables
 *****************************************************************************/

static uint8_t updateSeconds = TRUE;
static uint8_t shadowSeconds;
#ifndef ALWAYS_READ_RTC
static uint8_t oldSeconds;
#endif
static uint8_t redrawFirstTime;
static uint8_t timeToRedraw = FALSE;
static uint8_t clockAppActive = FALSE;
static volatile tRtcTime currTime;
char weekdayStrings[][10] = {"Sunday   ", "Monday   ", "Tuesday  ", "Wednesday", "Thursday ", "Friday   ", "Saturday "};
char monthStrings[][4] = {"", "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};

/******************************************************************************
 * Local functions
 *****************************************************************************/

#ifndef ALWAYS_READ_RTC
static tU8 i2bcd( tU8 n ) {
  return ( ((n / 10) << 4) | (n % 10) );
}
#endif

static tU8 bcd2i( tU8 bcd ) {
  return ( ((bcd >> 4) * 10) + (bcd & 0x0F) );
}

/******************************************************************************
 * Public functions
 *****************************************************************************/

uint8_t app_clock_1hz_tick(void)
{
  char timeString[10];

  //check if clock application active
  if (TRUE == clockAppActive)
  {
#ifdef ALWAYS_READ_RTC
    if (TRUE == updateSeconds)
      pcf8523_readTime((tRtcTime*)&currTime);
    else
    {
      shadowSeconds++;
      if (shadowSeconds >= 60)
      {
        pcf8523_readTime((tRtcTime*)&currTime);
        shadowSeconds = bcd2i((currTime.seconds)&0x7f);	//should normally be 0 but in case misaligned a correction will be made
        timeToRedraw = TRUE;
      }
    }
#else
    //update current time
    currTime.seconds = i2bcd(bcd2i(currTime.seconds & 0x7f) + 1);	//(currTime.seconds)++;
    if (bcd2i(currTime.seconds & 0x7f) >= 60)
    {
      currTime.seconds = 0;
      currTime.minutes = i2bcd(bcd2i(currTime.minutes) + 1);	//(currTime.minutes)++;
      if (bcd2i(currTime.minutes) >= 60)
      {
        currTime.minutes = 0;
        currTime.hours = i2bcd(bcd2i(currTime.hours) + 1);	//(currTime.hours)++;
        if (bcd2i(currTime.hours) >= 24)
        {
          currTime.hours = 0;

          //re-read RTC to get correct day, weekday, month and year
          pcf8523_readTime((tRtcTime*)&currTime);
        }
      }
    }
#endif //ALWAYS_READ_RTC

    if ((TRUE == updateSeconds) || ((FALSE == updateSeconds) && (TRUE == timeToRedraw)))
    {
      timeToRedraw = FALSE;

      //update LCD with new time
      lcd_copyImage16( 6, 24, &winfreesystem14x16_bits[(currTime.hours >> 4)<<4], 8, 16, FALSE);
      lcd_copyImage16(14, 24, &winfreesystem14x16_bits[(currTime.hours & 0x0f)<<4], 8, 16, FALSE);
      lcd_copyImage16(22, 24, &winfreesystem14x16_bits[(10)<<4], 4, 16, FALSE);

      lcd_copyImage16(26, 24, &winfreesystem14x16_bits[(currTime.minutes >> 4)<<4], 8, 16, FALSE);
      lcd_copyImage16(34, 24, &winfreesystem14x16_bits[(currTime.minutes & 0x0f)<<4], 8, 16, FALSE);

      if (TRUE == updateSeconds)
      {
        lcd_copyImage16(42, 24, &winfreesystem14x16_bits[(10)<<4], 4, 16, FALSE);
        lcd_copyImage16(46, 24, &winfreesystem14x16_bits[((currTime.seconds & 0x7f) >> 4)<<4], 8, 16, FALSE);
        lcd_copyImage16(54, 24, &winfreesystem14x16_bits[(currTime.seconds & 0x0f)<<4], 8, 16, FALSE);
      }

      lcd_putString(6, 44, weekdayStrings[currTime.weekdays], LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
      lcd_putString(6, 54, monthStrings[bcd2i(currTime.months)], LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
      timeString[0] = '0' + (currTime.days >> 4);
      timeString[1] = '0' + (currTime.days & 0x0f);
      timeString[2] = ' ';
      timeString[3] = '2';
      timeString[4] = '0';
      timeString[5] = '0' + (currTime.years >> 4);
      timeString[6] = '0' + (currTime.years & 0x0f);
      timeString[7] = '\0';
      lcd_putString(30, 54, timeString, LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
      lcd_update_vcom();
      if (((currTime.hours == 0x00) && (currTime.minutes == 0x00) && (currTime.seconds == 0x00)) ||
          (redrawFirstTime == TRUE))
    	  lcd_updateScreen(0, 95);	//update complete screen since weekday has changed
      else
    	  lcd_updateScreen(24, 40);  //update just section of screen with time
      redrawFirstTime = FALSE;
      return TRUE;
    }
    else
      return FALSE;
  }
  else
    return FALSE;
}

void applicationClock(void)
{
  pcf8523_registerCallback(app_clock_1hz_tick);

  /* Clear the LCD screen */
  lcd_clearScreen(LCD_COLOR_WHITE, TRUE);
  lcd_rect(1, 1, 94, 94, LCD_COLOR_BLACK);

  /****************************************************/
  /* Draw menu options                                */
  /****************************************************/
  lcd_putString(6,  4, "Clock App", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
  lcd_line     (5, 12, 58, 12, LCD_COLOR_BLACK, FALSE);
  lcd_line     (5, 14, 58, 14, LCD_COLOR_BLACK, FALSE);

  lcd_line     (2, 66, 93, 66, LCD_COLOR_BLACK, FALSE);
  lcd_putString(6, 68, "Return to menu", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
  lcd_putString(6, 76, "by pressing", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
  lcd_putString(6, 84, "'wakeup' key", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, TRUE);

#ifndef ALWAYS_READ_RTC
  //wait until new second begins
  pcf8523_readTime((tRtcTime*)&currTime);
  oldSeconds = bcd2i(currTime.seconds);
  while (bcd2i(currTime.seconds) == oldSeconds)
    pcf8523_readTime((tRtcTime*)&currTime);
#endif

  if (FALSE == updateSeconds)
  {
    pcf8523_readTime((tRtcTime*)&currTime);
    shadowSeconds = bcd2i(currTime.seconds & 0x7f);
    timeToRedraw = TRUE;
  }

  //put accelerometer to sleep
  adxl345_setPowerControl(0x00);

  //put capacitive sensor to sleep
  PCF8885_SLEEP_PIN_HIGH;

  clockAppActive = TRUE;

  prepareIoForSleep(KEEP_SPI0_ACTIVE, MAKE_CLKOUT_INACTIVE);

  redrawFirstTime = TRUE;

  while((LPC_GPIO->PIN[0] & (1<<16)))	//clock application runs until 'wakeup' key pressed
  {
    //enter sleep mode
    PMU_Sleep(MCU_POWER_DOWN, LPC_SYSCON->PDSLEEPCFG & ~(WDT_OSC_PD | BOD_PD));
  }
  clockAppActive = FALSE;
  pcf8523_registerCallback(NULL);

  //wakeup capacitive sensor again
  PCF8885_SLEEP_PIN_LOW;
}


void applicationSetClock(void)
{
  tRtcTime time;
  char ch[2] = {0};
  int32_t cursor = CURSOR_YEAR_1;
  uint8_t cursorChanged = TRUE;
  int32_t val = 0;
  uint8_t key = 0;
  uint8_t done = FALSE;
  uint32_t inactiveCnt = 0;

  /* Clear the LCD screen */
  lcd_clearScreen(LCD_COLOR_WHITE, TRUE);
  lcd_rect(1, 1, 94, 94, LCD_COLOR_BLACK);

  initSysTickCounter(10);

  pcf8523_readTime(&time);

  /****************************************************/
  /* Draw menu options                                */
  /****************************************************/
  lcd_putString(6,  4, "Set Clock App", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
  lcd_line     (5, 12, 82, 12, LCD_COLOR_BLACK, FALSE);
  lcd_line     (5, 14, 82, 14, LCD_COLOR_BLACK, FALSE);

  lcd_putString(6,  DATE_Y,    "Date: ", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
  lcd_putString(6,  TIME_Y,    "Time: ", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
  lcd_putString(6,  DAY_Y,     "Day: ", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
  lcd_putString(6,  UPD_SEC_Y, "Update each ", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
  lcd_putString(6,  UPD_SEC_Y+8, "second: ", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);


  while (!done) {

    sysTickDelay(CLOCK_SET_DELAY);

    key = pcf8885_readSense();

    if (inactiveCnt * CLOCK_SET_DELAY >= 60000) {
      // inactive for 60 s -> exit app
      done = TRUE;
      break;
    }

    if (key == 0 && !cursorChanged) {
      inactiveCnt++;
      continue;
    }

    inactiveCnt = 0;

    if (key == CAP_KEY_RIGHT) {
      cursor++;
      cursorChanged = TRUE;
    }
    else if (key == CAP_KEY_LEFT){
      cursor--;
      cursorChanged = TRUE;
    }
    if (cursor >= CURSOR_NUM) cursor = CURSOR_YEAR_1;
    if (cursor < 0) cursor = (CURSOR_NUM-1);


    // cursor has changed -> redraw all
    // more efficient would of course to only redraw what changed
    if (cursorChanged) {
      cursorChanged = FALSE;

      // year
      ch[0] = '0' + ((time.years >> 4));
      lcd_putString(38, DATE_Y, ch, LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
      ch[0] = '0' + ((time.years & 0x0f));
      lcd_putString(45, DATE_Y, ch, LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);

      ch[0] = '-';
      lcd_putString(52, DATE_Y, ch, LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);

      // month
      ch[0] = '0' + ((time.months >> 4));
      lcd_putString(59, DATE_Y, ch, LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
      ch[0] = '0' + ((time.months & 0x0f));
      lcd_putString(66, DATE_Y, ch, LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);

      ch[0] = '-';
      lcd_putString(73, DATE_Y, ch, LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);

      // day
      ch[0] = '0' + ((time.days >> 4));
      lcd_putString(80, DATE_Y, ch, LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
      ch[0] = '0' + ((time.days & 0x0f));
      lcd_putString(87, DATE_Y, ch, LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);

      // hours
      ch[0] = '0' + ((time.hours >> 4));
      lcd_putString(38, TIME_Y, ch, LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
      ch[0] = '0' + ((time.hours & 0x0f));
      lcd_putString(45, TIME_Y, ch, LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);

      ch[0] = ':';
      lcd_putString(52, TIME_Y, ch, LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);

      // minutes
      ch[0] = '0' + ((time.minutes >> 4));
      lcd_putString(59, TIME_Y, ch, LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
      ch[0] = '0' + ((time.minutes & 0x0f));
      lcd_putString(66, TIME_Y, ch, LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);

      // weekday
      lcd_putString(38, DAY_Y, weekdayStrings[time.weekdays], LCD_COLOR_BLACK,
           LCD_COLOR_WHITE, 0, FALSE);

      if (updateSeconds) {
        lcd_putString(48,  UPD_SEC_Y+8, "Yes ", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
      }
      else {
        lcd_putString(48,  UPD_SEC_Y+8, "No ", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
      }

      // OK button
      lcd_rect(6, BUTTON_Y, 45, BUTTON_Y+13, LCD_COLOR_BLACK);
      lcd_putString(21, BUTTON_Y+3, "OK", LCD_COLOR_BLACK,
           LCD_COLOR_WHITE, 0, FALSE);

      // Cancel button
      lcd_rect(49, BUTTON_Y, 90, BUTTON_Y+13, LCD_COLOR_BLACK);
      lcd_putString(53, BUTTON_Y+3, "Cancel", LCD_COLOR_BLACK,
           LCD_COLOR_WHITE, 0, TRUE);

    }

    switch (cursor) {
    case CURSOR_YEAR_1:
      val = time.years >> 4;
      if (key == CAP_KEY_UP) {
        val++;
      }
      else if (key == CAP_KEY_DOWN) {
        val--;
      }
      if (val > 9) val = 0;
      if (val < 0) val = 9;

      time.years = ((time.years & 0x0f) | (val << 4));

      ch[0] = '0' + val;
      lcd_putString(38, DATE_Y, ch, LCD_COLOR_WHITE, LCD_COLOR_BLACK, 0, TRUE);

      break;
    case CURSOR_YEAR_2:
      val = (time.years & 0x0f);
      if (key == CAP_KEY_UP) {
        val++;
      }
      else if (key == CAP_KEY_DOWN) {
        val--;
      }
      if (val > 9) val = 0;
      if (val < 0) val = 9;

      time.years = ((time.years & 0xf0) | (val & 0x0f));

      ch[0] = '0' + val;
      lcd_putString(45, DATE_Y, ch, LCD_COLOR_WHITE, LCD_COLOR_BLACK, 0, TRUE);
      break;
    case CURSOR_MONTH_1:
      val = (time.months >> 4);
      if (key == CAP_KEY_UP) {
        val++;
      }
      else if (key == CAP_KEY_DOWN) {
        val--;
      }
      if (val > 1) val = 0;
      if (val < 0) val = 1;

      time.months = ((time.months & 0x0f) | (val << 4));

      ch[0] = '0' + val;
      lcd_putString(59, DATE_Y, ch, LCD_COLOR_WHITE, LCD_COLOR_BLACK, 0, TRUE);
      break;
    case CURSOR_MONTH_2:
      val = (time.months & 0x0f);
      if (key == CAP_KEY_UP) {
        val++;
      }
      else if (key == CAP_KEY_DOWN) {
        val--;
      }
      if ((time.months >> 4) == 0) {
        if (val > 9) val = 0;
        if (val < 0) val = 9;
      }
      else {
        if (val > 2) val = 0;
        if (val < 0) val = 2;
      }

      time.months = ((time.months & 0xf0) | (val & 0x0f));

      ch[0] = '0' + val;
      lcd_putString(66, DATE_Y, ch, LCD_COLOR_WHITE, LCD_COLOR_BLACK, 0, TRUE);
      break;

    case CURSOR_DAY_1:
      val = (time.days >> 4);
      if (key == CAP_KEY_UP) {
        val++;
      }
      else if (key == CAP_KEY_DOWN) {
        val--;
      }

      // if February
      if (time.months == 0x02) {
        if (val > 2) val = 0;
        if (val < 0) val = 2;
      }
      else {
        if (val > 3) val = 0;
        if (val < 0) val = 3;
      }

      time.days = ((time.days & 0x0f) | (val << 4));

      ch[0] = '0' + val;
      lcd_putString(80, DATE_Y, ch, LCD_COLOR_WHITE, LCD_COLOR_BLACK, 0, TRUE);
      break;

    case CURSOR_DAY_2:
      val = (time.days & 0x0f);
      if (key == CAP_KEY_UP) {
        val++;
      }
      else if (key == CAP_KEY_DOWN) {
        val--;
      }
      if ((time.days >> 4) == 3) {
        if (val > 1) val = 0;
        if (val < 0) val = 1;
      }
      else {
        if (val > 9) val = 0;
        if (val < 0) val = 9;
      }

      time.days = ((time.days & 0xf0) | (val & 0x0f));

      ch[0] = '0' + val;
      lcd_putString(87, DATE_Y, ch, LCD_COLOR_WHITE, LCD_COLOR_BLACK, 0, TRUE);
      break;

    case CURSOR_HOUR_1:
      val = (time.hours >> 4);
      if (key == CAP_KEY_UP) {
        val++;
      }
      else if (key == CAP_KEY_DOWN) {
        val--;
      }

      if (val > 2) val = 0;
      if (val < 0) val = 2;


      time.hours = ((time.hours & 0x0f) | (val << 4));

      ch[0] = '0' + val;
      lcd_putString(38, TIME_Y, ch, LCD_COLOR_WHITE, LCD_COLOR_BLACK, 0, TRUE);
      break;

    case CURSOR_HOUR_2:
      val = (time.hours & 0x0f);
      if (key == CAP_KEY_UP) {
        val++;
      }
      else if (key == CAP_KEY_DOWN) {
        val--;
      }

      // 24 hour format
      if ((time.hours >> 4) == 2) {
        if (val > 3) val = 0;
        if (val < 0) val = 3;
      }
      else {
        if (val > 9) val = 0;
        if (val < 0) val = 9;
      }

      time.hours = ((time.hours & 0xf0) | (val & 0x0f));

      ch[0] = '0' + val;
      lcd_putString(45, TIME_Y, ch, LCD_COLOR_WHITE, LCD_COLOR_BLACK, 0, TRUE);
      break;

    case CURSOR_MIN_1:
      val = (time.minutes >> 4);
      if (key == CAP_KEY_UP) {
        val++;
      }
      else if (key == CAP_KEY_DOWN) {
        val--;
      }

      if (val > 5) val = 0;
      if (val < 0) val = 5;


      time.minutes = ((time.minutes & 0x0f) | (val << 4));

      ch[0] = '0' + val;
      lcd_putString(59, TIME_Y, ch, LCD_COLOR_WHITE, LCD_COLOR_BLACK, 0, TRUE);
      break;


    case CURSOR_MIN_2:
      val = (time.minutes & 0x0f);
      if (key == CAP_KEY_UP) {
        val++;
      }
      else if (key == CAP_KEY_DOWN) {
        val--;
      }

      if (val > 9) val = 0;
      if (val < 0) val = 9;

      time.minutes = ((time.minutes & 0xf0) | (val & 0x0f));

      ch[0] = '0' + val;
      lcd_putString(66, TIME_Y, ch, LCD_COLOR_WHITE, LCD_COLOR_BLACK, 0, TRUE);
      break;

    case CURSOR_WEEKDAY:
      val = time.weekdays;
      if (key == CAP_KEY_UP) {
        val++;
      }
      else if (key == CAP_KEY_DOWN) {
        val--;
      }

      if (val > 6) val = 0;
      if (val < 0) val = 6;

      time.weekdays = val;

      // clear area for week day (week days have different length)
      lcd_fillRect(38, DAY_Y, LCD_DISPLAY_WIDTH-3, DAY_Y+8, LCD_COLOR_WHITE);

      lcd_putString(38, DAY_Y, weekdayStrings[val], LCD_COLOR_WHITE, LCD_COLOR_BLACK, 0, TRUE);
      break;


    case CURSOR_UPD_SEC:

      if (key == CAP_KEY_UP || key == CAP_KEY_DOWN) {
        updateSeconds = !updateSeconds;
      }

      if(updateSeconds) {
        lcd_putString(48,  UPD_SEC_Y+8, "Yes", LCD_COLOR_WHITE, LCD_COLOR_BLACK, 0, TRUE);
      }
      else {
        lcd_putString(48,  UPD_SEC_Y+8, "No ", LCD_COLOR_WHITE, LCD_COLOR_BLACK, 0, TRUE);
      }
      break;

    case CURSOR_OK:

      lcd_putString(21, BUTTON_Y+3, "OK", LCD_COLOR_WHITE,
          LCD_COLOR_BLACK, 0, TRUE);

      if (key == CAP_KEY_OK) {
        pcf8523_setTime(&time);
        done = TRUE;
      }

      break;

    case CURSOR_CANCEL:

      lcd_putString(53, BUTTON_Y+3, "Cancel", LCD_COLOR_WHITE,
          LCD_COLOR_BLACK, 0, TRUE);

      if (key == CAP_KEY_OK) {
        done = TRUE;
      }

      break;

    }

  }


}


