/**********************************************************************
 *
 * Copyright(C) 2011, Embedded Artists AB
 * All rights reserved.
 *
 ***********************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 **********************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/

#ifdef __USE_CMSIS
#include "LPC11Uxx.h"
#endif

#include <stdio.h>

#include "type.h"
#include "general.h"
#include "application.h"
#include "gpio.h"
#include "adc.h"
#include "adxl345.h"
#include "s25fl004.h"
#include "ls013b4dn04.h"
#include "i2c.h"
#include "lm75b.h"
#include "pcf8885.h"
#include "pmu.h"
#include "pwrapi.h"
#include "clkconfig.h"
#include "wdt.h"

/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/

//Enable define to demonstrate lowest possible current consumption (about 2.14uA)
//#define DISABLE_WDT_WAKEUP

/******************************************************************************
 * Local variables
 *****************************************************************************/

static char messageString[] = "ADC=xxxx\ntemp=xx.xC\ncnt=xx";

/******************************************************************************
 * Local functions
 *****************************************************************************/

static void restoreIoAfterSleep(void)
{
  ADCInit(ADC_CLK);
  LPC_IOCON->PIO0_23 = (1<<0) | (2<<3) | (0<<7);
}

/******************************************************************************
 * Main function
 *****************************************************************************/

int main(void)
{
  LPC_SYSCON->PDRUNCFG &= ~(WDT_OSC_PD | ADC_PD);
  LPC_SYSCON->PDRUNCFG |= (SYS_OSC_PD);

  GPIOInit();

  prepareIoForSleep(MAKE_SPI0_INACTIVE, MAKE_CLKOUT_INACTIVE);

  //Set LED pin (PIO0_7) to output and turn LED off
  EA_LED_INIT;
  EA_LED_OFF;

  //Turn off on-chip USB peripheral block
  LPC_SYSCON->PDRUNCFG |= (1 << 10);	//Power-down USB PHY
  LPC_SYSCON->PDRUNCFG |= (1 <<  8);	//Power-down USB PLL
  //  LPC_SYSCON->PDRUNCFG |= (IRC_OUT_PD | IRC_PD);  //Power-down IRC

  PMU_Init();

  //Turn LCD on
  PWR_LCD_INIT;
  PWR_LCD_ON;

  //Turn SPI-FLASH off
  PWR_FLASH_INIT;
  PWR_FLASH_OFF;

  //Init and enable I2C pull-up resistors
  PWR_I2C_INIT;
  PWR_I2C_ON;

  if ( I2CInit( (uint32_t)I2CMASTER ) == FALSE )  /* initialize I2c */
  {
    while ( 1 );        /* Fatal error */
  }

  //put capacitive sensor to sleep
  pcf8885_init();
  PCF8885_SLEEP_PIN_HIGH;

  //Init accelerometer
  adxl345_init();
  adxl345_setPowerControl(0x00);

  //Init pin that controls voltage to trimming potentiometer
  PWR_ADC_INIT;
  PWR_ADC_OFF;

  /* Initialize ADC  */
  ADCInit( ADC_CLK );

  //Init the temperature sensor
#ifdef DISABLE_WDT_WAKEUP
  lm75b_config(LM75B_CFG_SHUTDOWN);
#else
  lm75b_config(LM75B_CFG_NORMAL);
#endif

  lcd_init();
  lcd_putString( 0,  4, "Demo of ADC,", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
  lcd_putString( 0, 12, "WDT and", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
  lcd_putString( 0, 20, "temp sensor.", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, TRUE);

  WDT_CLK_Setup(WDTCLK_SRC_WDT_OSC);
  WDTInit();
  WDTFeed();

  while(1)
  {
    static uint8_t cnt = 0;
    uint16_t tempVal, adcVal;

#ifdef DISABLE_WDT_WAKEUP
    prepareIoForSleep(MAKE_SPI0_INACTIVE, MAKE_CLKOUT_INACTIVE);

    //For lowest possible current consumption, ignore 1Hz RTC interrupt signal
    //Set PIO0_11 to be GPIO, output, low and have pullup disabled
    LPC_IOCON->TDI_PIO0_11 = (1<<0) | (0<<3) | (1<<7);
    LPC_GPIO->CLR[0]  =   1<<11;
    LPC_GPIO->DIR[0] &= ~(1<<11);
#else
    prepareIoForSleep(KEEP_SPI0_ACTIVE, MAKE_CLKOUT_INACTIVE);
#endif

    //enter sleep mode (MCU_SLEEP or MCU_DEEP_SLEEP or MCU_POWER_DOWN)
#ifdef DISABLE_WDT_WAKEUP
    PMU_Sleep(MCU_POWER_DOWN, (BOD_PD | WDT_OSC_PD)); //Power down BOD and WDT oscillator
#else
    PMU_Sleep(MCU_POWER_DOWN, (BOD_PD)); //Power down BOD (enabling WDT_OSC cost 1.7uA)
#endif

    //Wakeup after interrupt (watchdog trigger)
    WDTFeed();

    //Will actually need >1ms delay until ADC value can be read.
    //Reduce capacitor on ADC wire to get faster response time
    PWR_ADC_ON;

    restoreIoAfterSleep();

    //Read temperature & ADC and print out result
    tempVal = lm75b_readTemp();

    messageString[4] = '0' + (adcVal / 1000);
    messageString[5] = '0' + ((adcVal / 100) % 10);
    messageString[6] = '0' + ((adcVal / 10) % 10);
    messageString[7] = '0' + (adcVal % 10);

    cnt++;
    cnt = (cnt % 100);
    messageString[24] = '0' + (cnt / 10);
    messageString[25] = '0' + (cnt % 10);

    //read ADC input
    adcVal = ADCRead(7);
    PWR_ADC_OFF;

    messageString[14] = '0' + (tempVal+50)/1000;
    messageString[15] = '0' + ((tempVal+50)/100) % 10;
    messageString[17] = '0' + (((tempVal%100) + 5) / 10);

    lcd_update_vcom(); //Update VCOM with 1Hz
    lcd_putString( 0, 35, messageString, LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, TRUE);
  }

  return 0 ;
}


