/*----------------------------------------------------------------------------
 *      Name:    DEMO.C
 *      Purpose: USB HID Demo
 *      Version: V1.20
 *----------------------------------------------------------------------------
 *      This software is supplied "AS IS" without any warranties, express,
 *      implied or statutory, including but not limited to the implied
 *      warranties of fitness for purpose, satisfactory quality and
 *      noninfringement. Keil extends you a royalty-free right to reproduce
 *      and distribute executable files created using this software for use
 *      on NXP Semiconductors LPC microcontroller devices only. Nothing else 
 *      gives you the right to use this software.
 *
 * Copyright (c) 2009 Keil - An ARM Company. All rights reserved.
 *---------------------------------------------------------------------------*/
#include <string.h>
#include "LPC11Uxx.h"
#include "app_usbd_cfg.h"
#include "mw_usbd_rom_api.h"
#include "power_api.h"
#include "uart.h"

#include <cr_section_macros.h>
#include <NXP/crp.h>

extern ErrorCode_t usb_hid_init(USBD_HANDLE_T hUsb, USB_INTERFACE_DESCRIPTOR* pIntfDesc, uint32_t* mem_base, uint32_t* mem_size);
extern ErrorCode_t USB_Configure_Event (USBD_HANDLE_T hUsb);
extern uint8_t USB_DeviceDescriptor[];
extern uint8_t USB_StringDescriptor[];
extern uint8_t USB_FsConfigDescriptor[];

// Variable to store CRP value in. Will be placed automatically
// by the linker when "Enable Code Read Protect" selected.
// See crp.h header for more information
__CRP const unsigned int CRP_WORD = CRP_NO_CRP ;




#include "type.h"
#include "demo.h"
#include "i2c.h"
#include "gpio.h"
#include "adc.h"

/**********************************************************************
 ** Global data
**********************************************************************/
USBD_API_T* pUsbApi;

/* local data */
static USBD_HANDLE_T hUsb;


void USB_IRQHandler(void)
{
  pUsbApi->hw->ISR(hUsb);
}

void USB_pin_clk_init(void)
{
  /* Enable AHB clock to the GPIO domain. */
  LPC_SYSCON->SYSAHBCLKCTRL |= (1<<6);

  /* Enable AHB clock to the USB block and USB RAM. */
  LPC_SYSCON->SYSAHBCLKCTRL |= ((0x1<<14)|(0x1<<27));

  /* Pull-down is needed, or internally, VBUS will be floating. This is to
  address the wrong status in VBUSDebouncing bit in CmdStatus register (applies to rev A) */
  LPC_IOCON->PIO0_3   &= ~0x1F;
//  LPC_IOCON->PIO0_3   |= ((0x1<<3)|(0x01<<0));	/* Secondary function VBUS */
  LPC_IOCON->PIO0_3   |= (0x01<<0);			/* Secondary function VBUS */
  LPC_IOCON->PIO0_6   &= ~0x07;
  LPC_IOCON->PIO0_6   |= (0x01<<0);			/* Secondary function SoftConn */

  return;
}


/*
 *  Get HID Input Report -> InReport
 */

void GetInReport (void) {
  if ( LPC_GPIO->PIN[0] & (1<<16) ) {		/* Check if PBINT is pressed */
    InReport = 0x01;
  } else {
    InReport = 0x00;
  }
  return;
}


/*
 *  Set HID Output Report <- OutReport
 */

void SetOutReport (void) {
  /* port1, bit 14~17 are used as LED indication from
  HID utility. */
//  LPC_GPIO->PIN[1] &= ~(0x0F<<14);
//  LPC_GPIO->PIN[1] |= (OutReport<<14);

  LPC_GPIO->PIN[0] &= ~(0x01<<7);
  LPC_GPIO->PIN[0] |= (OutReport<<7);

  return; 
}

#include "application.h"
#include "ls013b4dn04.h"
#include "adxl345.h"
#include "s25fl004.h"
#include "lm75b.h"
#include "pcf8523.h"
#include "pcf8885.h"
#include "systick.h"



static uint8_t logRow = 0;
static void logResult(char* test, uint8_t result)
{
  char* r = "PASS";

  lcd_putString(6, 4+logRow*8, test, LCD_COLOR_BLACK,
      LCD_COLOR_WHITE, 25, TRUE);

  if (!result) {
    r = "FAIL";
  }

  lcd_putString(64, 4+logRow*8, r, LCD_COLOR_BLACK,
      LCD_COLOR_WHITE, 25, TRUE);

  logRow++;
}

#define INFO1_ROW (9)
#define INFO2_ROW (10)

static void logInfo1(char* msg)
{
  lcd_fillRect(6, 4+INFO1_ROW*8, LCD_DISPLAY_WIDTH, 4+INFO1_ROW*8+8, LCD_COLOR_WHITE);
  lcd_putString(6, 4+INFO1_ROW*8, msg, LCD_COLOR_BLACK,
      LCD_COLOR_WHITE, 25, TRUE);
}

static uint8_t test_acc(void)
{
  uint8_t id = 0;
  adxl345_init();

  id = adxl345_getDeviceID();

  return (id == 0xe5);
}

static uint8_t test_flash(void)
{
  // Testing the flash ID. Init function will return
  // FALSE if the ID is wrong...
  return (uint8_t)s25fl004_init();
}

static uint8_t test_temp(void)
{
#define TEMP_MIN 1500
#define TEMP_MAX 3700
  int32_t t = 0;

  //Init the temperature sensor
  lm75b_config(LM75B_CFG_NORMAL);
  t =  lm75b_readTemp();
  t =  lm75b_readTemp();
  t =  lm75b_readTemp();
  t =  lm75b_readTemp();


  return (t > TEMP_MIN && t < TEMP_MAX);
}

static uint8_t test_rtc(void)
{
  uint8_t result = TRUE;
  tRtcTime time;
  pcf8523_init();

  time.seconds = 0;
/*  time.minutes = 0;
  time.hours = 0;
  time.days = 0;
  time.weekdays = 0;
  time.months = 0;
  time.years = 0;
*/

  do {
    if (!pcf8523_setTime(&time)) {
      result = FALSE;
      break;
    }

    sysTickDelay(1100);

    if (!pcf8523_readTime(&time)) {
      result = FALSE;
      break;
    }

    if (time.seconds < 1 || time.seconds > 2) {
      result = FALSE;
    }


  } while(0);

  return result;
}

static uint8_t test_trimpot(void)
{
#define TP_DELAY_MS 50
#define TP_TOTAL_TIME_MS 10000
#define TP_LOW 15
#define TP_HIGH 1000

  uint32_t adcVal = 0;
  uint32_t low = 9999;
  uint32_t high = 0;
  uint8_t result = FALSE;
  uint32_t time = 0;

  PWR_ADC_INIT; //ADC power control pin
  PWR_ADC_OFF;

  /* Initialize ADC  */
  ADCInit( ADC_CLK );

  logInfo1("Run: Trimpot");

  while (1) {

    PWR_ADC_ON;
    adcVal = ADCRead(7);
    PWR_ADC_OFF;

    if (adcVal < low) {
      low = adcVal;
    }

    if (adcVal > high) {
      high = adcVal;
    }

    if (low <= TP_LOW && high >= TP_HIGH) {
      result = TRUE;
      break;
    }

    sysTickDelay(TP_DELAY_MS);
    time += TP_DELAY_MS;

    if(time > TP_TOTAL_TIME_MS) {
      result = FALSE;
      break;
    }
  }

  return result;
}

static uint8_t test_capsense(void)
{
#define CP_DELAY_MS (20)
#define CP_TOTAL_TIME_MS (10000)

  uint8_t keys = 0;
  uint8_t lastKeys = 0;
  uint8_t val = 0;
  uint32_t time = 0;
  uint8_t result = FALSE;

  logInfo1("Run: CapSense");

  // clear info row 2
  lcd_fillRect(6, 4+INFO2_ROW*8, LCD_DISPLAY_WIDTH, 4+INFO2_ROW*8+8,
      LCD_COLOR_WHITE);

  pcf8885_init();
  while (1) {
    val = pcf8885_readSense();
    keys |= val;

    if (keys == 0xFF) {
      result = TRUE;
      break;
    }

    if (lastKeys != keys) {
      lastKeys = keys;

      if (keys & 0x01) {
        lcd_putString(6, 4+INFO2_ROW*8, "1", LCD_COLOR_BLACK,
            LCD_COLOR_WHITE, 25, TRUE);
      }
      if (keys & 0x02) {
        lcd_putString(6+8, 4+INFO2_ROW*8, "2", LCD_COLOR_BLACK,
            LCD_COLOR_WHITE, 25, TRUE);
      }
      if (keys & 0x04) {
        lcd_putString(6+16, 4+INFO2_ROW*8, "3", LCD_COLOR_BLACK,
            LCD_COLOR_WHITE, 25, TRUE);
      }
      if (keys & 0x08) {
        lcd_putString(6+24, 4+INFO2_ROW*8, "4", LCD_COLOR_BLACK,
            LCD_COLOR_WHITE, 25, TRUE);
      }
      if (keys & 0x10) {
        lcd_putString(6+32, 4+INFO2_ROW*8, "5", LCD_COLOR_BLACK,
            LCD_COLOR_WHITE, 25, TRUE);
      }
      if (keys & 0x20) {
        lcd_putString(6+40, 4+INFO2_ROW*8, "6", LCD_COLOR_BLACK,
            LCD_COLOR_WHITE, 25, TRUE);
      }
      if (keys & 0x40) {
        lcd_putString(6+48, 4+INFO2_ROW*8, "7", LCD_COLOR_BLACK,
            LCD_COLOR_WHITE, 25, TRUE);
      }
      if (keys & 0x80) {
        lcd_putString(6+56, 4+INFO2_ROW*8, "8", LCD_COLOR_BLACK,
            LCD_COLOR_WHITE, 25, TRUE);
      }


    }

    sysTickDelay(CP_DELAY_MS);
    time += CP_DELAY_MS;

    if(time > CP_TOTAL_TIME_MS) {
      result = FALSE;
      break;
    }

  }

  // clear info row 2
  lcd_fillRect(6, 4+INFO2_ROW*8, LCD_DISPLAY_WIDTH, 4+INFO2_ROW*8+8,
      LCD_COLOR_WHITE);

  return result;
}

static uint8_t test_buttons(void)
{
#define BT_DELAY_MS (50)
#define BT_TOTAL_TIME_MS (10000)
  uint8_t wakeup = 0;
  uint8_t isp = 0;
  uint8_t result = FALSE;
  uint32_t time = 0;

  logInfo1("Run: Buttons");
  LPC_GPIO->DIR[0] &= ~(0x1<<16);
  LPC_GPIO->DIR[0] &= ~(0x1<<1);

  while (1) {
    if ((LPC_GPIO->PIN[0] & (1<<16)) == 0) {
      wakeup = 1;
    }

    if ((LPC_GPIO->PIN[0] & (1<<1)) == 0) {
      isp = 1;
    }

    if (wakeup && isp) {
      result = TRUE;
      break;
    }

    sysTickDelay(BT_DELAY_MS);
    time += BT_DELAY_MS;

    if(time > BT_TOTAL_TIME_MS) {
      result = FALSE;
      break;
    }
  }


  return result;
}

#include "pic01.h"
#include "pic02.h"
#include "pic03.h"
#include "pic04.h"
#include "pic05.h"
#include "pic06.h"
#include "pic07.h"
#include "pic08.h"
#include "pic09.h"
#include "pic10.h"

static const char* files[] =
{
    NULL,      /* first section reserved */
    &pic01[0], /* first image */
    &pic02[0],
    &pic03[0],
    &pic04[0],
    &pic05[0],
    &pic06[0],
    &pic07[0],
    &pic08[0],
    &pic09[0],
    &pic10[0],
};

static const int fileSizes[] =
{
    0,
    sizeof(pic01),
    sizeof(pic02),
    sizeof(pic03),
    sizeof(pic04),
    sizeof(pic05),
    sizeof(pic06),
    sizeof(pic07),
    sizeof(pic08),
    sizeof(pic09),
    sizeof(pic10),
};

static uint8_t flash_images(void)
{
  uint32_t len = 0;
  int i = 0;
  uint8_t result = TRUE;
  uint32_t sectorSize = 0;

  do {

    sectorSize = flash_getSectorSize();

    for (i = 0; i < sizeof(files)/sizeof(unsigned char*); i++) {
      if (files[i] != NULL) {
        len = fileSizes[i];

        if (len > sectorSize) {
          result = FALSE;
          break;
        }

        flash_erase_sector(i*sectorSize);
        flash_write((uint8_t*)files[i], i*sectorSize, len);

      }
    }

  } while (0);

  return result;
}

int main (void) 
{
#if POWERDOWN_MODE_USB_WAKEUP
  uint32_t i, j;
#endif
  volatile uint8_t result = 0;
  volatile uint8_t testOk = 1;

  /* It's very important that POWERDOWN_MODE_USB_WAKEUP and REMOTE_WAKEUP_ENABLE
  flags should not be turned on at the same time!!!!!! */
    
  SystemCoreClockUpdate();


  LPC_GPIO->DIR[0] |= (0x1<<7);

  LPC_GPIO->DIR[0] &= ~(0x1<<1);

  PWR_I2C_INIT; //I2C pull-up control pin
  PWR_I2C_ON;

  if ( I2CInit( (uint32_t)I2CMASTER ) == FALSE )  /* initialize I2c */
  {
    while ( 1 );        /* Fatal error */
  }

  initSysTickCounter(100);

  // Init the LCD
  lcd_init();

  lcd_clearScreen(LCD_COLOR_WHITE, FALSE);

  result = test_acc();
  logResult("Acc", result);
  testOk &= result;

  result = test_flash();
  logResult("Flash", result);
  testOk &= result;

  result = test_temp();
  logResult("Temp", result);
  testOk &= result;

  result = test_rtc();
  logResult("RTC", result);
  testOk &= result;

  result = test_trimpot();
  logResult("Trimpot", result);
  testOk &= result;

  result = test_capsense();
  logResult("Cap", result);
  testOk &= result;

  result = test_buttons();
  logResult("Button", result);
  testOk &= result;

  result = flash_images();
  logResult("Images", result);
  testOk &= result;

  if (testOk) {
    logInfo1("Result PASSED");
  }
  else {
    logInfo1("Result FAILED");
  }

  {
    USBD_API_INIT_PARAM_T usb_param;
  	ErrorCode_t ret;
    USB_CORE_DESCS_T desc;

    SystemCoreClockUpdate();

    /* get USB API table pointer */
    pUsbApi = (USBD_API_T*)((*(ROM **)(0x1FFF1FF8))->pUSBD);

    /* enable clocks and pinmux for usb */
    USB_pin_clk_init();

    /* initilize call back structures */
    memset((void*)&usb_param, 0, sizeof(USBD_API_INIT_PARAM_T));
    usb_param.usb_reg_base = LPC_USB_BASE;
    usb_param.mem_base = 0x10001000;
    usb_param.mem_size = 0x1000;
    usb_param.max_num_ep = 2;
    usb_param.USB_Configure_Event = USB_Configure_Event;

  /* Initialize Descriptor pointers */
    memset((void*)&desc, 0, sizeof(USB_CORE_DESCS_T));
    desc.device_desc = (uint8_t *)&USB_DeviceDescriptor[0];
    desc.string_desc = (uint8_t *)&USB_StringDescriptor[0];
    desc.full_speed_desc = (uint8_t *)&USB_FsConfigDescriptor[0];
    desc.high_speed_desc = (uint8_t *)&USB_FsConfigDescriptor[0];

    /* USB Initialization */
    ret = pUsbApi->hw->Init(&hUsb, &desc, &usb_param);
    if (ret == LPC_OK)
    {
      ret = usb_hid_init(hUsb, (USB_INTERFACE_DESCRIPTOR *)&USB_FsConfigDescriptor[sizeof(USB_CONFIGURATION_DESCRIPTOR)],
  			&usb_param.mem_base, &usb_param.mem_size);
      if (ret == LPC_OK) {
        NVIC_EnableIRQ(USB_IRQn); //  enable USB interrrupts
        /* now connect */
        pUsbApi->hw->Connect(hUsb, 1);
      }
    }
  }

  while (1);

}
