/**********************************************************************
 *
 * Copyright(C) 2011, Embedded Artists AB
 * All rights reserved.
 *
 ***********************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 **********************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/

#ifdef __USE_CMSIS
#include "LPC11Uxx.h"
#endif

#include <stdio.h>

#include "type.h"
#include "general.h"
#include "application.h"
#include "systick.h"
#include "gpio.h"
#include "adc.h"
#include "ls013b4dn04.h"
#include "s25fl004.h"
#include "i2c.h"
#include "pcf8523.h"
#include "pcf8885.h"
#include "adxl345.h"
#include "lm75b.h"
#include "pwrapi.h"
#include "pmu.h"
#include "lpc_winfreesystem14x16.h"

/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/

#define ALWAYS_READ_RTC
#define FLASH_WITH_LED
#define UPDATE_SECONDS

/******************************************************************************
 * Local variables
 *****************************************************************************/

static uint8_t shadowSeconds;
static uint8_t oldSeconds;
static uint8_t timeToRedraw = FALSE;
static volatile tRtcTime currTime;
static tRtcTime setTime;
static char weekdayStrings[][10] = {"Sunday   ", "Monday   ", "Tuesday  ", "Wednesday", "Thursday ", "Friday   ", "Saturday "};
static char monthStrings[][4] = {"", "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};

/******************************************************************************
 * Local functions
 *****************************************************************************/

static tU8 i2bcd( tU8 n ) {
  return ( ((n / 10) << 4) | (n % 10) );
}
static tU8 bcd2i( tU8 bcd ) {
  return ( ((bcd >> 4) * 10) + (bcd & 0x0F) );
}

static void restoreIoAfterSleep(void)
{
  SSP_IOConfig(0);
  SSP_Init(0);
}

/******************************************************************************
 * Main function
 *****************************************************************************/

int main(void)
{
  LPC_SYSCON->PDRUNCFG &= ~(WDT_OSC_PD | ADC_PD);
  LPC_SYSCON->PDRUNCFG |= (SYS_OSC_PD);

  GPIOInit();

  prepareIoForSleep(MAKE_SPI0_INACTIVE, MAKE_CLKOUT_INACTIVE);

  //Set LED pin (PIO0_7) to output and turn LED off
  EA_LED_INIT;
  EA_LED_OFF;

  //Turn on-chip USB peripheral block off
  LPC_SYSCON->PDRUNCFG |= (1 << 10);	//Power-down USB PHY
  LPC_SYSCON->PDRUNCFG |= (1 <<  8);	//Power-down USB PLL
  //  LPC_SYSCON->PDRUNCFG |= (IRC_OUT_PD | IRC_PD);  //Power-down IRC

  LPC_SYSCON->PDRUNCFG &= ~(WDT_OSC_PD);

  LPC_SYSCON->STARTERP0 = (0x1<<1);			//Flex IRQ1 (RTC irq) can startup mcu
  LPC_SYSCON->STARTERP1 = (1<<12) | (1<<13);	//WDT and BOD can startup mcu

  //Turn LCD on
  PWR_LCD_INIT;
  PWR_LCD_ON;

  //Turn SPI-FLASH off
  PWR_FLASH_INIT;
  PWR_FLASH_OFF;

  //Init and enable I2C pull-up resistors
  PWR_I2C_INIT;
  PWR_I2C_ON;

  if ( I2CInit( (uint32_t)I2CMASTER ) == FALSE )  /* initialize I2c */
  {
    while ( 1 );        /* Fatal error */
  }

  //put capacitive sensor to sleep
  pcf8885_init();
  PCF8885_SLEEP_PIN_HIGH;

  //Init accelerometer
  adxl345_init();
  adxl345_setPowerControl(0x00);

  //Init pin that controls voltage to trimming potentiometer
  PWR_ADC_INIT;
  PWR_ADC_OFF;

  //Init the temperature sensor in power down mode
  lm75b_config(LM75B_CFG_SHUTDOWN);

  //Init the RTC
  pcf8523_init();

  lcd_init();
  lcd_rect(1, 1, 94, 94, LCD_COLOR_BLACK);
  lcd_putString(6,  4, "RTC demo", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
  lcd_line     (5, 12, 52, 12, LCD_COLOR_BLACK, FALSE);
  lcd_line     (5, 14, 52, 14, LCD_COLOR_BLACK, FALSE);
  lcd_putString(3, 80, "1Hz wakeup irq", LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, TRUE);

#ifndef ALWAYS_READ_RTC
  //wait until new second begins
  pcf8523_readTime((tRtcTime*)&currTime);
  oldSeconds = bcd2i(currTime.seconds);
  while (bcd2i(currTime.seconds) == oldSeconds)
    pcf8523_readTime((tRtcTime*)&currTime);
#endif

#ifndef UPDATE_SECONDS
  pcf8523_readTime((tRtcTime*)&currTime);
  shadowSeconds = bcd2i(currTime.seconds & 0x7f);
  timeToRedraw = TRUE;
#endif

  //Enter forever loop...
  while(1)
  {
    prepareIoForSleep(MAKE_SPI0_INACTIVE, MAKE_CLKOUT_INACTIVE);
    PMU_Sleep(MCU_POWER_DOWN, (BOD_PD | WDT_OSC_PD)); //Power down BOD and WDT oscillator
    restoreIoAfterSleep();

    //Check if rtc interrupt (is the only one possible, but check anyways)
    if (TRUE == rtcFlag)
    {
      volatile uint8_t dummy;
      char timeString[10];

      rtcFlag = FALSE;
#ifdef FLASH_WITH_LED
      EA_LED_ON;
#endif

#ifdef ALWAYS_READ_RTC
#ifdef UPDATE_SECONDS
      pcf8523_readTime((tRtcTime*)&currTime);
#else
      shadowSeconds++;
      if (shadowSeconds >= 60)
      {
        pcf8523_readTime((tRtcTime*)&currTime);
        shadowSeconds = bcd2i((currTime.seconds)&0x7f);	//should normally be 0 but in case misaligned a correction will be made
        timeToRedraw = TRUE;
      }
#endif
#else
      //update current time
      currTime.seconds = i2bcd(bcd2i(currTime.seconds & 0x7f) + 1);	//(currTime.seconds)++;
      if (bcd2i(currTime.seconds & 0x7f) >= 60)
      {
        currTime.seconds = 0;
        currTime.minutes = i2bcd(bcd2i(currTime.minutes) + 1);	//(currTime.minutes)++;
        if (bcd2i(currTime.minutes) >= 60)
        {
          currTime.minutes = 0;
          currTime.hours = i2bcd(bcd2i(currTime.hours) + 1);	//(currTime.hours)++;
          if (bcd2i(currTime.hours) >= 24)
          {
            currTime.hours = 0;

            //re-read RTC to get correct day, weekday, month and year
            pcf8523_readTime((tRtcTime*)&currTime);
          }
        }
      }
#endif //ALWAYS_READ_RTC

#ifdef UPDATE_SECONDS
      if (1)
#else
        if (TRUE == timeToRedraw)
#endif
        {
          timeToRedraw = FALSE;

          //update LCD with new time
          lcd_copyImage16( 6, 24, &winfreesystem14x16_bits[(currTime.hours >> 4)<<4], 8, 16, FALSE);
          lcd_copyImage16(14, 24, &winfreesystem14x16_bits[(currTime.hours & 0x0f)<<4], 8, 16, FALSE);
          lcd_copyImage16(22, 24, &winfreesystem14x16_bits[(10)<<4], 4, 16, FALSE);

          lcd_copyImage16(26, 24, &winfreesystem14x16_bits[(currTime.minutes >> 4)<<4], 8, 16, FALSE);
          lcd_copyImage16(34, 24, &winfreesystem14x16_bits[(currTime.minutes & 0x0f)<<4], 8, 16, FALSE);

#ifdef UPDATE_SECONDS
          lcd_copyImage16(42, 24, &winfreesystem14x16_bits[(10)<<4], 4, 16, FALSE);
          lcd_copyImage16(46, 24, &winfreesystem14x16_bits[((currTime.seconds & 0x7f) >> 4)<<4], 8, 16, FALSE);
          lcd_copyImage16(54, 24, &winfreesystem14x16_bits[(currTime.seconds & 0x0f)<<4], 8, 16, FALSE);
#endif

          lcd_putString(6, 48, weekdayStrings[currTime.weekdays], LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
          lcd_putString(6, 58, monthStrings[bcd2i(currTime.months)], LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);
          timeString[0] = '0' + (currTime.days >> 4);
          timeString[1] = '0' + (currTime.days & 0x0f);
          timeString[2] = ' ';
          timeString[3] = '2';
          timeString[4] = '0';
          timeString[5] = '0' + (currTime.years >> 4);
          timeString[6] = '0' + (currTime.years & 0x0f);
          timeString[7] = '\0';
          lcd_putString(30, 58, timeString, LCD_COLOR_BLACK, LCD_COLOR_WHITE, 0, FALSE);

          lcd_update_vcom();
          if ((currTime.hours == 0x00) && (currTime.minutes == 0x00) && (currTime.seconds == 0x00))
        	  lcd_updateScreen(0, 95);	//update complete screen since weekday has changed
          else
        	  lcd_updateScreen(24, 40);  //update just section of screen with time
        }
        else
          //Update VCOM on LCD
          lcd_isr();

#ifdef FLASH_WITH_LED
      EA_LED_OFF;
#endif
    }

    //check if wakeup-key is pressed
    if ((LPC_GPIO->PIN[0] & (1<<16)) == 0)
    {
      //set new time
      setTime.seconds = 0;
      setTime.minutes = 0x24;
      setTime.hours = 0x22;
      setTime.days = 0x21;
      setTime.weekdays = 0x03;
      setTime.months = 0x09;
      setTime.years = 0x11;
      pcf8523_setTime(&setTime);
    }
  }

  return 0 ;
}



