/**********************************************************************
 *
 * Copyright(C) 2011, Embedded Artists AB
 * All rights reserved.
 *
 ***********************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 **********************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/

#include "LPC11Uxx.h"
#include "type.h"
#include "general.h"
#include "application.h"
#include "gpio.h"
#include "i2c.h"
#include "adxl345.h"

/******************************************************************************
 * Defines and typedefs
 *****************************************************************************/
#define ADXL345_I2C_ADDR	(0x1D)

/******************************************************************************
 * External global variables
 *****************************************************************************/

/******************************************************************************
 * Local variables
 *****************************************************************************/

/******************************************************************************
 * Local Functions
 *****************************************************************************/

static uint8_t
SingleByteRead(uint8_t regAddr)
{
	tU8 retVal;
	I2C_write_read(ADXL345_I2C_ADDR, &regAddr, 1, &retVal, 1);
	return retVal;

}

static int
SingleByteWrite(uint8_t address, uint8_t data){
	uint8_t tx[2];
   tx[0] = address;
   tx[1] = data;
   return I2C_write_read(ADXL345_I2C_ADDR, tx, 2, NULL, 0);
}

static void
multiByteRead(uint8_t address, uint8_t* output, uint16_t size) {
	I2C_write_read(ADXL345_I2C_ADDR, &address, 1, output, size);
}


static int multiByteWrite(uint8_t address, uint8_t* ptr_data, uint16_t size) {
	uint8_t tx[9];
	tx[0] = address;
	tx[1] = ptr_data[0];
	tx[2] = ptr_data[1];
	tx[3] = ptr_data[2];
	tx[4] = ptr_data[3];
	tx[5] = ptr_data[4];
	tx[6] = ptr_data[5];
	tx[7] = ptr_data[6];
	tx[8] = ptr_data[7];
	return I2C_write_read(ADXL345_I2C_ADDR, tx, size, NULL, 0);

   //        int ack;
//
//               ack = i2c_.write( ADXL345_I2C_WRITE, &address, 1);  //tell it where to write to
//        return ack | i2c_.write( ADXL345_I2C_READ, ptr_data, size);  //tell it what data to write

}


/******************************************************************************
 * Public Functions
 *****************************************************************************/
void adxl345_getOutput(uint16_t* readings){
	uint8_t buffer[6];
    multiByteRead(ADXL345_DATAX0_REG, buffer, 6);

    readings[0] = (uint16_t)buffer[1] << 8 | (uint16_t)buffer[0];
    readings[1] = (uint16_t)buffer[3] << 8 | (uint16_t)buffer[2];
    readings[2] = (uint16_t)buffer[5] << 8 | (uint16_t)buffer[4];
}

uint8_t adxl345_getDeviceID() {
	 return SingleByteRead(ADXL345_DEVID_REG);
}

uint32_t adxl345_setPowerMode(uint8_t mode) {

    //Get the current register contents, so we don't clobber the rate value.
	uint8_t registerContents = (mode << 4) | SingleByteRead(ADXL345_BW_RATE_REG);

   return SingleByteWrite(ADXL345_BW_RATE_REG, registerContents);

}

uint8_t adxl345_getPowerControl() {
    return SingleByteRead(ADXL345_POWER_CTL_REG);
}

uint32_t adxl345_setPowerControl(uint8_t settings) {
    return SingleByteWrite(ADXL345_POWER_CTL_REG, settings);

}



uint8_t adxl345_getDataFormatControl(void){

    return SingleByteRead(ADXL345_DATA_FORMAT_REG);
}

uint32_t adxl345_setDataFormatControl(uint8_t settings){

   return SingleByteWrite(ADXL345_DATA_FORMAT_REG, settings);

}

uint32_t adxl345_setDataRate(uint8_t rate) {

    //Get the current register contents, so we don't clobber the power bit.
	uint8_t registerContents = SingleByteRead(ADXL345_BW_RATE_REG);

    registerContents &= 0x10;
    registerContents |= rate;

    return SingleByteWrite(ADXL345_BW_RATE_REG, registerContents);

}


uint8_t adxl345_getOffset(uint8_t axis) {

	uint8_t address = 0;

    if (axis == ADXL345_X) {
        address = ADXL345_OFSX_REG;
    } else if (axis == ADXL345_Y) {
        address = ADXL345_OFSY_REG;
    } else if (axis == ADXL345_Z) {
        address = ADXL345_OFSZ_REG;
    }

   return SingleByteRead(address);
}

uint32_t adxl345_setOffset(uint8_t axis, uint8_t offset) {

	uint8_t address = 0;

    if (axis == ADXL345_X) {
        address = ADXL345_OFSX_REG;
    } else if (axis == ADXL345_Y) {
        address = ADXL345_OFSY_REG;
    } else if (axis == ADXL345_Z) {
        address = ADXL345_OFSZ_REG;
    }

   return SingleByteWrite(address, offset);

}


uint8_t adxl345_getFifoControl(void){

    return SingleByteRead(ADXL345_FIFO_CTL);

}

uint32_t adxl345_setFifoControl(uint8_t settings){
   return SingleByteWrite(ADXL345_FIFO_STATUS, settings);

}

uint8_t adxl345_getFifoStatus(void){

    return SingleByteRead(ADXL345_FIFO_STATUS);

}



uint8_t adxl345_getTapThreshold(void) {

    return SingleByteRead(ADXL345_THRESH_TAP_REG);
}

uint32_t adxl345_setTapThreshold(uint8_t threshold) {

   return SingleByteWrite(ADXL345_THRESH_TAP_REG, threshold);

}


uint32_t adxl345_getTapDuration(void) {

    return (int)SingleByteRead(ADXL345_DUR_REG)*625;
}

uint32_t adxl345_setTapDuration(uint16_t duration_us) {

	uint16_t tapDuration = duration_us / 625;
    uint8_t tapChar[2];
     tapChar[0] = (tapDuration & 0x00FF);
     tapChar[1] = (tapDuration >> 8) & 0x00FF;
    return multiByteWrite(ADXL345_DUR_REG, tapChar, 2);

}

uint32_t adxl345_getTapLatency(void) {

    return (uint16_t)SingleByteRead(ADXL345_LATENT_REG); //TODO!!!  *1.25;
}

uint32_t adxl345_setTapLatency(uint16_t latency_ms) {

//    latency_ms = latency_ms / 1.25;
    uint8_t latChar[2];
     latChar[0] = (latency_ms & 0x00FF);
     latChar[1] = (latency_ms << 8) & 0xFF00;
    return multiByteWrite(ADXL345_LATENT_REG, latChar, 2);

}

uint32_t adxl345_getWindowTime(void) {

    return ((uint32_t)SingleByteRead(ADXL345_WINDOW_REG)*125)/100;
}

uint32_t adxl345_setWindowTime(uint16_t window_ms) {

//    window_ms = (window_ms * 8) / 10; // = / 1.25;
    uint8_t windowChar[2];
    windowChar[0] = (window_ms & 0x00FF);
    windowChar[1] = ((window_ms << 8) & 0xFF00);
   return multiByteWrite(ADXL345_WINDOW_REG, windowChar, 2);

}

uint8_t adxl345_getActivityThreshold(void) {

    return SingleByteRead(ADXL345_THRESH_ACT_REG);
}

uint32_t adxl345_setActivityThreshold(uint8_t threshold) {
    return SingleByteWrite(ADXL345_THRESH_ACT_REG, threshold);

}

uint8_t adxl345_getInactivityThreshold(void) {
    return SingleByteRead(ADXL345_THRESH_INACT_REG);

}

uint32_t adxl345_setInactivityThreshold(uint8_t threshold) {
    return SingleByteWrite(ADXL345_THRESH_INACT_REG, threshold);

}

uint8_t adxl345_getTimeInactivity(void) {

    return SingleByteRead(ADXL345_TIME_INACT_REG);

}

uint32_t adxl345_setTimeInactivity(uint8_t timeInactivity) {
    return SingleByteWrite(ADXL345_TIME_INACT_REG, timeInactivity);

}

uint8_t adxl345_getActivityInactivityControl(void) {

    return SingleByteRead(ADXL345_ACT_INACT_CTL_REG);

}

uint32_t adxl345_setActivityInactivityControl(uint8_t settings) {
    return SingleByteWrite(ADXL345_ACT_INACT_CTL_REG, settings);

}

uint8_t adxl345_getFreefallThreshold(void) {

    return SingleByteRead(ADXL345_THRESH_FF_REG);

}

uint32_t adxl345_setFreefallThreshold(uint8_t threshold) {
   return SingleByteWrite(ADXL345_THRESH_FF_REG, threshold);

}

uint8_t adxl345_getFreefallTime(void) {

    return SingleByteRead(ADXL345_TIME_FF_REG)*5;

}

uint32_t adxl345_setFreefallTime(uint16_t freefallTime_ms) {
     freefallTime_ms = freefallTime_ms / 5;
     uint8_t fallChar[2];
     fallChar[0] = (freefallTime_ms & 0x00FF);
     fallChar[1] = (freefallTime_ms << 8) & 0xFF00;

    return multiByteWrite(ADXL345_TIME_FF_REG, fallChar, 2);

}

uint8_t adxl345_getTapAxisControl(void) {

    return SingleByteRead(ADXL345_TAP_AXES_REG);

}

uint32_t adxl345_setTapAxisControl(uint8_t settings) {
   return SingleByteWrite(ADXL345_TAP_AXES_REG, settings);

}

uint8_t adxl345_getTapSource(void) {

    return SingleByteRead(ADXL345_ACT_TAP_STATUS_REG);

}



uint8_t adxl345_getInterruptEnableControl(void) {

    return SingleByteRead(ADXL345_INT_ENABLE_REG);

}

uint32_t adxl345_setInterruptEnableControl(uint8_t settings) {
   return SingleByteWrite(ADXL345_INT_ENABLE_REG, settings);

}

uint8_t adxl345_getInterruptMappingControl(void) {

    return SingleByteRead(ADXL345_INT_MAP_REG);

}

uint32_t adxl345_setInterruptMappingControl(uint8_t settings) {
    return SingleByteWrite(ADXL345_INT_MAP_REG, settings);

}

uint8_t adxl345_getInterruptSource(void){

    return SingleByteRead(ADXL345_INT_SOURCE_REG);

}

void adxl345_init(void)
{
	// initialize the BW data rate
	//value greater than or equal to 0x0A is written into the rate bits (Bit D3 through Bit D0) in the BW_RATE register
	SingleByteWrite(ADXL345_BW_RATE_REG, ADXL345_1600HZ);

	//Data format (for +-16g) - This is done by setting Bit D3 of the DATA_FORMAT register (Address 0x31) and writing a value of 0x03 to the range bits (Bit D1 and Bit D0) of the DATA_FORMAT register (Address 0x31).
	// full res and +_16g
	SingleByteWrite(ADXL345_DATA_FORMAT_REG, 0x0B);

	// Set Offset  - programmed into the OFSX, OFSY, and OFXZ registers, respectively, as 0xFD, 0x03 and 0xFE.
	SingleByteWrite(ADXL345_OFSX_REG, 0xFD);
	SingleByteWrite(ADXL345_OFSY_REG, 0x03);
	SingleByteWrite(ADXL345_OFSZ_REG, 0xFE);

	//Set INT1/INT2 as input (PIO0_13/14)
	GPIOSetDir( 0, 13, 0 );
	GPIOSetDir( 0, 14, 0 );
}

/*****************************************************************************
 ** Function name:		FLEX_INT0_IRQHandler
 **
 ** Descriptions:		Use GPIO0_13 pin as interrupt source
 **
 ** parameters:			None
 **
 ** Returned value:		None
 **
 *****************************************************************************/
void FLEX_INT0_IRQHandler(void)
{
	PWR_SIGNAL_IRQ_START;
	if ( LPC_GPIO_PIN_INT->IST & (0x1<<0) )
	{
		if ( LPC_GPIO_PIN_INT->FALL & (0x1<<0) )
		{
			accelFlag = TRUE;
			PWR_I2C_ON;
			accelIrqSrc = adxl345_getInterruptSource();
			PWR_I2C_OFF;
			LPC_GPIO_PIN_INT->FALL = 0x1<<0;
		}
		LPC_GPIO_PIN_INT->IST = 0x1<<0;
	}
	PWR_SIGNAL_IRQ_END;
	return;
}

void adxl345_init_irq(void)
{
	//initialize P0.13 as edge sensitive interrupt, active falling edge
	GPIOSetFlexInterrupt( CHANNEL0, PORT0, 13, 0, 0 );
	NVIC_SetPriority(FLEX_INT0_IRQn, 2);
	LPC_SYSCON->STARTERP0 |= (0x1<<0);  //This interrupt can wakup the mcu from power down mode
	GPIOFlexIntEnable( CHANNEL0, 0 );
}
