/**********************************************************************
 *
 * Copyright(C) 2011, Embedded Artists AB
 * All rights reserved.
 *
 ***********************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 **********************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/

#include "type.h"
#include "ls013b4dn04.h"
#include "gpio.h"
#include "font5x7.h"
#include "systick.h"

/******************************************************************************
 * Defines and typedefs
 *****************************************************************************/


//defines the VCOM bit in the command word that goes to the LCD
#define VCOM_HI 0x40
#define VCOM_LO 0x00
#define MLCD_WR 0x80 //MLCD write line command
#define MLCD_CM 0x20 //MLCD clear memory command
#define MLCD_NO 0x00 //MLCD NOP command (used to switch VCOM)

typedef struct
{
  uint8_t lineNum;
  uint8_t pixels[12];
  uint8_t trailer;
} tLineBuf;
typedef struct
{
  uint8_t  command;
  tLineBuf line[96];
  uint8_t  trailer;
} tFrameBuf;

#define LCD_DISP_INIT	GPIOSetDir( 0, 8, 1 )
#define LCD_DISP_OFF	GPIOSetBitValue( 0, 8, 0 )
#define LCD_DISP_ON		GPIOSetBitValue( 0, 8, 1 )

#define BMP_TYPE (0x4D42)

typedef struct bmpHeader {
  uint16_t type;
  uint32_t size;
  uint32_t reserved;
  uint32_t offset;
  uint32_t headerSize;
  uint32_t width;
  uint32_t height;
  uint16_t plane;
  uint16_t bits;
  uint32_t compression;
  uint32_t imageSize;
  uint32_t xres;
  uint32_t yres;
  uint32_t ncolors;
  uint32_t nimportant_colors;
} __attribute__((packed))  bmpHeader_t;


/******************************************************************************
 * Local variables
 *****************************************************************************/

static tFrameBuf frameBuf;
static uint8_t shortLcdCommand[3] = {0,0,0};
static uint8_t const font_mask[8] = {0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01};

static uint8_t vcom;
static uint8_t updateVcom;
static uint8_t clrScreen;
uint8_t lcdSpiActive;
static uint8_t lcdActive = FALSE;

/******************************************************************************
 * Local Functions
 *****************************************************************************/

static uint8_t bitReverse(uint8_t bData)
{
  uint8_t lookup[16] = { 0, 8, 4, 12, 2, 10, 6, 14, 1, 9, 5, 13, 3, 11, 7, 15 };
  return (uint8_t)(((lookup[(bData & 0x0F)]) << 4) + lookup[((bData & 0xF0) >> 4)]);
}

/******************************************************************************
 *
 * Description:
 *    Draw a horizontal line
 *
 * Params:
 *   [in] x0 - start x position
 *   [in] y0 - y position
 *   [in] x1 - end y position
 *   [in] color - color of the pixel
 *
 *****************************************************************************/
static void hLine(uint8_t x0, uint8_t y0, uint8_t x1, lcd_color_t color)
{
  // TODO: optimize

  uint8_t bak;

  if (x0 > x1)
  {
    bak = x1;
    x1 = x0;
    x0 = bak;
  }

  while(x1 >= x0)
  {
    lcd_putPixel(x0, y0, color);
    x0++;
  }
}

/******************************************************************************
 *
 * Description:
 *    Draw a vertical line
 *
 * Params:
 *   [in] x0 - x position
 *   [in] y0 - start y position
 *   [in] y1 - end y position
 *   [in] color - color of the pixel
 *
 *****************************************************************************/
static void vLine(uint8_t x0, uint8_t y0, uint8_t y1, lcd_color_t color)
{
  uint8_t bak;

  if(y0 > y1)
  {
    bak = y1;
    y1 = y0;
    y0 = bak;
  }

  while(y1 >= y0)
  {
    lcd_putPixel(x0, y0, color);
    y0++;
  }
  return;
}

/******************************************************************************
 * Public Functions
 *****************************************************************************/

/******************************************************************************
 *
 * Description:
 *    Initialize the ls013b4dn04 Driver
 *
 *****************************************************************************/
void lcd_init (void)
{
  uint8_t i;

  LCD_DISP_INIT;
  LCD_DISP_OFF;
  PWR_LCD_INIT;
  PWR_LCD_ON;

  //init SPI for LCD on SPI channel#0
  SSP_IOConfig(0);
  SSP_Init(0);

  //initialize data structure for frame buffer
  for(i=0; i<96; i++)
  {
    frameBuf.line[i].lineNum = bitReverse(i+1);
    frameBuf.line[i].trailer = 0x00;
  }
  frameBuf.trailer = 0x00;

  lcdSpiActive = FALSE;
  updateVcom = FALSE;
  clrScreen = 0;

  lcd_clearScreen(LCD_COLOR_WHITE, TRUE);	//clear screen

  LCD_DISP_ON;

  //start 1 Hz update
  lcdActive = TRUE;
}


void lcd_clearScreen (lcd_color_t color, uint8_t updateFlag)
{
  uint8_t i,j;
  uint8_t c = 0;

  if (color == LCD_COLOR_WHITE)
    c = 0xff;

  //initialize data structure for frame buffer
  for(i=0; i<96; i++)
  {
    frameBuf.line[i].lineNum = bitReverse(i+1);
    for(j=0; j<12; j++)
      frameBuf.line[i].pixels[j] = c;
  }

  if (updateFlag)
  {
    if (color == LCD_COLOR_WHITE)
    {
      //check so that spi communication with the display is not active
      if (FALSE == lcdSpiActive)
      {
        SPI0_SSEL_HIGH;
        SPI0_SSEL_HIGH;
        //wait 6 us

        shortLcdCommand[0] = vcom | MLCD_CM;	//pos 1&2 are always zero since initialization
        SSP_Send( 0, shortLcdCommand, 3);

        //wait 2us
        SPI0_SSEL_HIGH;
        SPI0_SSEL_HIGH;
        SPI0_SSEL_LOW;
      }
      else
        clrScreen = MLCD_CM;
    }
    else
      lcd_updateScreen(0, 95);
  }
}

void lcd_update_vcom (void)
{
  //toggle vcom bit
  if (vcom == VCOM_HI) vcom = VCOM_LO;
  else vcom = VCOM_HI;
}

/******************************************************************************
 *
 * Description:
 *    Interrupt Service Routine (IRS) for the ls013b4dn04 LCD.
 *    Toggle VCOM bit every time this routine is called.
 *
 *****************************************************************************/
void lcd_isr (void)
{
  if (lcdActive)
  {
    //toggle vcom bit
    if (vcom == VCOM_HI) vcom = VCOM_LO;
    else vcom = VCOM_HI;

    //check so that spi communication with the display is not active
    if (FALSE == lcdSpiActive)
    {
      SPI0_SSEL_HIGH;
      //wait 6 us
      SPI0_SSEL_HIGH;
      SPI0_SSEL_HIGH;
      SPI0_SSEL_HIGH;

      shortLcdCommand[0] = vcom;	//pos 1&2 are always zero since initialization
      SSP_Send( 0, shortLcdCommand, 3);

      //wait 2us
      SPI0_SSEL_HIGH;
      SPI0_SSEL_HIGH;
      SPI0_SSEL_HIGH;
      SPI0_SSEL_LOW;
    }
    else
      updateVcom = TRUE;
  }
}


void lcd_updateScreen(uint8_t startLine, uint8_t endLine)
{
  lcdSpiActive = TRUE;
  SPI0_SSEL_HIGH;
  SPI0_SSEL_HIGH;	//wait 6 us
  SPI0_SSEL_HIGH;

  //check if all of screen shall be updated
  if ((startLine == 0) && (endLine == 95))
  {
    frameBuf.command = MLCD_WR | vcom;
    SSP_Send( 0, (uint8_t *)&frameBuf, sizeof(frameBuf));
  }
  else if (startLine == 0)
  {
    frameBuf.command = MLCD_WR | vcom;
    frameBuf.line[endLine+1].lineNum = 0; //Add temporary trailer
    SSP_Send( 0, (uint8_t *)&frameBuf, (sizeof(tLineBuf)*(endLine+1)) + 2);
    frameBuf.line[endLine+1].lineNum = bitReverse(endLine+2); //Restore line number
  }
  else
  {
    frameBuf.command = MLCD_WR | vcom;
    frameBuf.line[startLine-1].trailer = MLCD_WR | vcom; //Add temporary command
    frameBuf.line[endLine+1].lineNum = 0; //Add temporary trailer
    SSP_Send( 0, (uint8_t *)&(frameBuf.line[startLine-1].trailer), (sizeof(tLineBuf)*(endLine-startLine+1)) + 2);
    frameBuf.line[startLine-1].trailer = 0;	//Restore trailer
    frameBuf.line[endLine+1].lineNum = bitReverse(endLine+2); //Restore line number
  }

  //wait 2us
  SPI0_SSEL_HIGH;
  SPI0_SSEL_HIGH;
  SPI0_SSEL_HIGH;
  SPI0_SSEL_LOW;
  lcdSpiActive = FALSE;

  //check if toggle VCOM
  if ((TRUE == updateVcom) || clrScreen)
  {
	lcdSpiActive = TRUE;
    updateVcom = FALSE;

    //wait 2 us
    SPI0_SSEL_LOW;
    SPI0_SSEL_LOW;
    SPI0_SSEL_LOW;
    SPI0_SSEL_HIGH;
    //wait 6 us
    SPI0_SSEL_HIGH;
    SPI0_SSEL_HIGH;

    shortLcdCommand[0] = vcom | clrScreen;	//pos 1&2 are always zero since initialization
    clrScreen = 0;
    SSP_Send( 0, shortLcdCommand, 3);

    //wait 2us
    SPI0_SSEL_HIGH;
    SPI0_SSEL_HIGH;
    SPI0_SSEL_LOW;
    lcdSpiActive = FALSE;
  }
}

/******************************************************************************
 *
 * Description:
 *    Draw one pixel on the display
 *
 * Params:
 *   [in] x - x position
 *   [in] y - y position
 *   [in] color - color of the pixel
 *
 *****************************************************************************/
void lcd_putPixel(uint8_t x, uint8_t y, lcd_color_t color) {
  uint8_t add;
  uint8_t hAddr;
  uint8_t mask;

  if (x > LCD_DISPLAY_WIDTH) {
    return;
  }
  if (y > LCD_DISPLAY_HEIGHT) {
    return;
  }

  hAddr = x / 8;

  // Calculate mask from rows basically do a x%8 and remainder is bit position
  add = x & 0x7;
  mask = 0x80 >> add;             // Right shift 0x80 by bit position

  if(color > 0)
    frameBuf.line[y].pixels[hAddr] &= ~mask;
  else
    frameBuf.line[y].pixels[hAddr] |= mask;

}

/******************************************************************************
 *
 * Description:
 *    Draw a line on the display starting at x0,y0 and ending at x1,y1
 *
 * Params:
 *   [in] x0 - start x position
 *   [in] y0 - start y position
 *   [in] x1 - end x position
 *   [in] y1 - end y position
 *   [in] color - color of the line
 *
 *****************************************************************************/
void lcd_line(uint8_t x0, uint8_t y0, uint8_t x1, uint8_t y1, lcd_color_t color, uint8_t updateFlag)
{
  int16_t   dx = 0, dy = 0;
  int8_t    dx_sym = 0, dy_sym = 0;
  int16_t   dx_x2 = 0, dy_x2 = 0;
  int16_t   di = 0;

  dx = x1-x0;
  dy = y1-y0;


  if(dx == 0)           /* vertical line */
  {
    vLine(x0, y0, y1, color);
    if (updateFlag)
    	lcd_updateScreen(0, 95);
    return;
  }

  if(dx > 0)
  {
    dx_sym = 1;
  }
  else
  {
    dx_sym = -1;
  }


  if(dy == 0)           /* horizontal line */
  {
    hLine(x0, y0, x1, color);
    if (updateFlag)
    	lcd_updateScreen(0, 95);
    return;
  }


  if(dy > 0)
  {
    dy_sym = 1;
  }
  else
  {
    dy_sym = -1;
  }

  dx = dx_sym*dx;
  dy = dy_sym*dy;

  dx_x2 = dx*2;
  dy_x2 = dy*2;

  if(dx >= dy)
  {
    di = dy_x2 - dx;
    while(x0 != x1)
    {

      lcd_putPixel(x0, y0, color);
      x0 += dx_sym;
      if(di<0)
      {
        di += dy_x2;
      }
      else
      {
        di += dy_x2 - dx_x2;
        y0 += dy_sym;
      }
    }
    lcd_putPixel(x0, y0, color);
  }
  else
  {
    di = dx_x2 - dy;
    while(y0 != y1)
    {
      lcd_putPixel(x0, y0, color);
      y0 += dy_sym;
      if(di < 0)
      {
        di += dx_x2;
      }
      else
      {
        di += dx_x2 - dy_x2;
        x0 += dx_sym;
      }
    }
    lcd_putPixel(x0, y0, color);
  }
  if (updateFlag)
	  lcd_updateScreen(0, 95);
  return;
}

/******************************************************************************
 *
 * Description:
 *    Draw a circle on the display starting at x0,y0 with radius r
 *
 * Params:
 *   [in] x0 - start x position
 *   [in] y0 - start y position
 *   [in] r - radius
 *   [in] color - color of the circle
 *
 *****************************************************************************/
void lcd_circle(uint8_t x0, uint8_t y0, uint8_t r, lcd_color_t color, uint8_t updateFlag)
{
  int16_t draw_x0, draw_y0;
  int16_t draw_x1, draw_y1;
  int16_t draw_x2, draw_y2;
  int16_t draw_x3, draw_y3;
  int16_t draw_x4, draw_y4;
  int16_t draw_x5, draw_y5;
  int16_t draw_x6, draw_y6;
  int16_t draw_x7, draw_y7;
  int16_t xx, yy;
  int16_t  di;

  if(r == 0)          /* no radius */
  {
    return;
  }

  draw_x0 = draw_x1 = x0;
  draw_y0 = draw_y1 = y0 + r;
  if(draw_y0 < LCD_DISPLAY_HEIGHT)
  {
    lcd_putPixel(draw_x0, draw_y0, color);     /* 90 degree */
  }

  draw_x2 = draw_x3 = x0;
  draw_y2 = draw_y3 = y0 - r;
  if(draw_y2 >= 0)
  {
    lcd_putPixel(draw_x2, draw_y2, color);    /* 270 degree */
  }

  draw_x4 = draw_x6 = x0 + r;
  draw_y4 = draw_y6 = y0;
  if(draw_x4 < LCD_DISPLAY_WIDTH)
  {
    lcd_putPixel(draw_x4, draw_y4, color);     /* 0 degree */
  }

  draw_x5 = draw_x7 = x0 - r;
  draw_y5 = draw_y7 = y0;
  if(draw_x5>=0)
  {
    lcd_putPixel(draw_x5, draw_y5, color);     /* 180 degree */
  }

  if(r == 1)
  {
	if (updateFlag)
		  lcd_updateScreen(0, 95);
    return;
  }

  di = 3 - 2*r;
  xx = 0;
  yy = r;
  while(xx < yy)
  {

    if(di < 0)
    {
      di += 4*xx + 6;
    }
    else
    {
      di += 4*(xx - yy) + 10;
      yy--;
      draw_y0--;
      draw_y1--;
      draw_y2++;
      draw_y3++;
      draw_x4--;
      draw_x5++;
      draw_x6--;
      draw_x7++;
    }
    xx++;
    draw_x0++;
    draw_x1--;
    draw_x2++;
    draw_x3--;
    draw_y4++;
    draw_y5++;
    draw_y6--;
    draw_y7--;

    if( (draw_x0 <= LCD_DISPLAY_WIDTH) && (draw_y0>=0) )
    {
      lcd_putPixel(draw_x0, draw_y0, color);
    }

    if( (draw_x1 >= 0) && (draw_y1 >= 0) )
    {
      lcd_putPixel(draw_x1, draw_y1, color);
    }

    if( (draw_x2 <= LCD_DISPLAY_WIDTH) && (draw_y2 <= LCD_DISPLAY_HEIGHT) )
    {
      lcd_putPixel(draw_x2, draw_y2, color);
    }

    if( (draw_x3 >=0 ) && (draw_y3 <= LCD_DISPLAY_HEIGHT) )
    {
      lcd_putPixel(draw_x3, draw_y3, color);
    }

    if( (draw_x4 <= LCD_DISPLAY_WIDTH) && (draw_y4 >= 0) )
    {
      lcd_putPixel(draw_x4, draw_y4, color);
    }

    if( (draw_x5 >= 0) && (draw_y5 >= 0) )
    {
      lcd_putPixel(draw_x5, draw_y5, color);
    }
    if( (draw_x6 <= LCD_DISPLAY_WIDTH) && (draw_y6 <= LCD_DISPLAY_HEIGHT) )
    {
      lcd_putPixel(draw_x6, draw_y6, color);
    }
    if( (draw_x7 >= 0) && (draw_y7 <= LCD_DISPLAY_HEIGHT) )
    {
      lcd_putPixel(draw_x7, draw_y7, color);
    }
  }
  if (updateFlag)
	  lcd_updateScreen(0, 95);
  return;
}

/******************************************************************************
 *
 * Description:
 *    Draw a rectangle on the display starting at x0,y0 and ending at x1,y1
 *
 * Params:
 *   [in] x0 - start x position
 *   [in] y0 - start y position
 *   [in] x1 - end x position
 *   [in] y1 - end y position
 *   [in] color - color of the rectangle
 *
 *****************************************************************************/
void lcd_rect(uint8_t x0, uint8_t y0, uint8_t x1, uint8_t y1, lcd_color_t color)
{
  hLine(x0, y0, x1, color);
  hLine(x0, y1, x1, color);
  vLine(x0, y0, y1, color);
  vLine(x1, y0, y1, color);
  lcd_updateScreen(0, 95);
}

/******************************************************************************
 *
 * Description:
 *    Fill a rectangle on the display starting at x0,y0 and ending at x1,y1
 *
 * Params:
 *   [in] x0 - start x position
 *   [in] y0 - start y position
 *   [in] x1 - end x position
 *   [in] y1 - end y position
 *   [in] color - color of the rectangle
 *
 *****************************************************************************/
void lcd_fillRect(uint8_t x0, uint8_t y0, uint8_t x1, uint8_t y1, lcd_color_t color)
{
  uint8_t i = 0;

  if(x0 > x1)
  {
    i  = x0;
    x0 = x1;
    x1 = i;
  }

  if(y0 > y1)
  {
    i  = y0;
    y0 = y1;
    y1 = i;
  }

  if(y0 == y1)
  {
    hLine(x0, y0, x1, color);
    return;
  }

  if(x0 == x1)
  {
    vLine(x0, y0, y1, color);
    return;
  }

  while(y0 <= y1)
  {
    hLine(x0, y0, x1, color);
    y0++;
  }
  lcd_updateScreen(0, 95);
  return;
}

uint8_t lcd_putChar(uint8_t x, uint8_t y, uint8_t ch, lcd_color_t fb, lcd_color_t bg)
{
  unsigned char data = 0;
  unsigned char i = 0, j = 0;

  lcd_color_t color = LCD_COLOR_BLACK;

  if((x >= (LCD_DISPLAY_WIDTH - 8)) || (y >= (LCD_DISPLAY_HEIGHT - 8)) )
  {
    return 0;
  }

  if( (ch < 0x20) || (ch > 0x7f) )
  {
    ch = 0x20;      /* unknown character will be set to blank */
  }

  ch -= 0x20;
  for(i=0; i<8; i++)
  {
    data = font5x7[ch][i];
    for(j=0; j<6; j++)
    {
      if( (data&font_mask[j])==0 )
      {
        color = bg;
      }
      else
      {
        color = fb;
      }
      lcd_putPixel(x, y, color);
      x++;
    }
    y++;
    x -= 6;
  }
  return( 1 );
}

void lcd_putString(uint8_t x, uint8_t y, char *pStr, lcd_color_t fb, lcd_color_t bg, uint8_t msDelay, uint8_t updateFlag)
{
  while(1)
  {
    if( (*pStr)=='\0' )
    {
      break;
    }
    if( (*pStr)=='\n' )
    {
      y += 8;
      x = 0;
      pStr++;
    }
    else if( lcd_putChar(x, y, *pStr++, fb, bg) == 0 )
    {
      break;
    }
    else
    {
      x += 6;
      if (msDelay)
      {
        lcd_updateScreen(0, 95);
        sysTickDelay(msDelay);
      }
    }
  }
  if (TRUE == updateFlag)
    lcd_updateScreen(0, 95);
  return;
}

void lcd_copyImage16(uint8_t x, uint8_t y, const uint16_t * img, uint8_t width, uint8_t height, uint8_t updateFlag)
{
  uint8_t colCnt;
  uint8_t rowCnt;
  uint16_t pixels = 0;

  for (rowCnt = 0; rowCnt < height; rowCnt++)
  {
    for (colCnt = 0; colCnt < width; colCnt++)
    {
      if ((colCnt & 0x000f) == 0)
        pixels = *img++;
      if (pixels & 0x8000)
        lcd_putPixel(x + colCnt, y, LCD_COLOR_BLACK);
      else
        lcd_putPixel(x + colCnt, y, LCD_COLOR_WHITE);
      pixels <<= 1;

    }
    y++;
  }
  if (updateFlag)
    lcd_updateScreen(0, 95);
}

void lcd_copyImage8(uint8_t x, uint8_t y, const uint8_t * img, uint8_t width, uint8_t height,
    uint8_t flipY, uint8_t updateFlag)
{
  uint8_t colCnt;
  uint8_t rowCnt;
  uint16_t pixels = 0;
  uint8_t y0 = 0;

  for (rowCnt = 0; rowCnt < height; rowCnt++)
  {
    for (colCnt = 0; colCnt < width; colCnt++)
    {
      if ((colCnt & 0x0007) == 0) {
        pixels = *img++;
      }

      if (flipY) {
        y0 = LCD_DISPLAY_HEIGHT-1-y;
      }
      else {
        y0 = y;
      }

      if (pixels & 0x80) {
        lcd_putPixel(x + colCnt, y0, LCD_COLOR_BLACK);
      }
      else {
        lcd_putPixel(x + colCnt, y0, LCD_COLOR_WHITE);
      }
      pixels <<= 1;

    }
    y++;
  }
  if (updateFlag)
    lcd_updateScreen(0, 95);
}

void lcd_copyImageFull(const uint8_t * img, uint8_t flipY, uint8_t updateFlag)
{
  uint8_t colCnt;
  uint8_t rowCnt;
  uint8_t y0 = 0;

  for (rowCnt = 0; rowCnt < 96; rowCnt++)
  {
      if (flipY) {
    	  y0 = 95-rowCnt;
      }
      else {
    	  y0 = rowCnt;
      }

      for (colCnt = 0; colCnt < 12; colCnt++)
      {
    	  frameBuf.line[y0].pixels[colCnt] = ~(*img++);
      }
  }
  if (updateFlag)
	  lcd_updateScreen(0, 95);
}


uint32_t lcd_getBmpOffset(uint8_t* bmp, uint16_t* pOff)
{
  bmpHeader_t * hdr = (bmpHeader_t *)bmp;

  // must be a BMP file
  if (hdr->type != BMP_TYPE) {
    return FALSE;
  }

  // currently only supports images that exactly fit the display
  if (hdr->width != LCD_DISPLAY_WIDTH || hdr->height != LCD_DISPLAY_HEIGHT) {
    return FALSE;
  }

  // only support one bit per pixel
  if (hdr->bits != 1) {
    return FALSE;
  }

  // only supports uncompressed images
  if (hdr->compression != 0) {
    return FALSE;
  }

  *pOff = hdr->offset;

  return TRUE;
}

uint32_t lcd_copyBmp(uint8_t * bmp, uint8_t updateFlag)
{

  uint16_t offset = 0;

  if (!lcd_getBmpOffset(bmp, &offset)) {
    return FALSE;
  }

  lcd_copyImage8(0, 0, &bmp[offset], 96, 96, TRUE, updateFlag);

  return TRUE;

}
