/**********************************************************************
 *
 * Copyright(C) 2011, Embedded Artists AB
 * All rights reserved.
 *
 ***********************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 **********************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/

#include "LPC11Uxx.h"
#include "type.h"
#include "application.h"
#include "gpio.h"
#include "i2c.h"
#include "pcf8523.h"
#include "ls013b4dn04.h"


/******************************************************************************
 * Defines and typedefs
 *****************************************************************************/
#define PCF8523_I2C_ADDR	(0x68)

#define RTC_CRYSTAL_IS_7PF
//#define FLASH_WITH_LED

/******************************************************************************
 * External global variables
 *****************************************************************************/

/******************************************************************************
 * Local variables
 *****************************************************************************/
static uint8_t (*pcf8523_pFunc)(void) = NULL;


/******************************************************************************
 * Local Functions
 *****************************************************************************/

/******************************************************************************
 * Public Functions
 *****************************************************************************/

/******************************************************************************
 *
 * Description:
 *    Read CTRL_" register
 *
 *****************************************************************************/
uint8_t pcf8523_readCTRL2 (void)
{
  uint8_t cmd;
  uint8_t retVal;

  cmd = PCF8523_REG_CONTROL_2; //Set address pointer to first register
  //I2C_write_read(i2c-addr,      *pBuf, len,  *pBuf2, len2
  I2C_write_read(PCF8523_I2C_ADDR, &cmd,   1, &retVal,   1);

  return retVal;
}

/******************************************************************************
 *
 * Description:
 *    Read CTRL_" register
 *
 *****************************************************************************/
int8_t pcf8523_writeCTRL2 (uint8_t regVal)
{
  uint8_t cmd[2];

  cmd[0] = PCF8523_REG_CONTROL_2; //Set address pointer to first register
  cmd[1] = regVal;

  //I2C_write_read(i2c-addr,      *pBuf, len, *pBuf2, len2
  return I2C_write_read(PCF8523_I2C_ADDR, cmd,   2,   NULL,   0);
}

/******************************************************************************
 *
 * Description:
 *    Read current time
 *
 * Params: None
 *
 * Returns:
 *   The current time
 *
 *****************************************************************************/
int8_t pcf8523_readTime(tRtcTime *pTime)
{
  uint8_t cmd = PCF8523_REG_SECONDS;

  //     I2C_write_read(i2c-addr,        *pBuf, len, *pBuf2, len2
  return I2C_write_read(PCF8523_I2C_ADDR, &cmd,   1,  (uint8_t*)pTime,   7);
}

/******************************************************************************
 *
 * Description:
 *    Set the time
 *
 * Params: None
 *
 * Returns:
 *   The current time
 *
 *****************************************************************************/
int8_t pcf8523_setTime(tRtcTime *pTime)
{
  uint8_t cmd[8];
  uint8_t *pU8 = (uint8_t *)pTime;
  uint32_t i;

  cmd[0] = PCF8523_REG_SECONDS;
  for(i=1; i<=8; i++)
    cmd[i] = *pU8++;

  //     I2C_write_read(i2c-addr,        *pBuf, len, *pBuf2, len2
  return I2C_write_read(PCF8523_I2C_ADDR, &cmd[0],   8,  NULL,   0);
}

/*****************************************************************************
 ** Function name:		FLEX_INT1_IRQHandler
 **
 ** Descriptions:		Use GPIO0_11 pin as interrupt source
 **
 ** parameters:			None
 **
 ** Returned value:		None
 **
 *****************************************************************************/
void FLEX_INT1_IRQHandler(void)
{
#ifdef FLASH_WITH_LED
EA_LED_ON;
#endif
  PWR_SIGNAL_IRQ_START;
  if ( LPC_GPIO_PIN_INT->IST & (0x1<<1) )
  {
    if ( LPC_GPIO_PIN_INT->FALL & (0x1<<1) )
    {
      rtcFlag = TRUE;

      //clear interrupt condition in RTC
//      volatile uint8_t val;
//      val = pcf8523_readCTRL2();
//      pcf8523_writeCTRL2(0);

      if (FALSE == lcdSpiActive)
      {
    	  //SSP interface has been disabled during sleep.
    	  //Needs to be reinitialized in order to send 1Hz signal to LCD
//    	  SSP_IOConfig(0);
//    	  SSP_Init(0);
      }

      //call callback if registered
      if (NULL != pcf8523_pFunc)
      {
        if (FALSE == pcf8523_pFunc())
          //Update VCOM on LCD
          lcd_isr();
      }
      else
          //Update VCOM on LCD
          lcd_isr();

      LPC_GPIO_PIN_INT->FALL = 0x1<<1;
    }
    LPC_GPIO_PIN_INT->IST = 0x1<<1;
  }
  PWR_SIGNAL_IRQ_END;
#ifdef FLASH_WITH_LED
  EA_LED_OFF;
#endif

  return;
}

/******************************************************************************
 *
 * Description:
 *    Register a callback function
 *
 *****************************************************************************/
void pcf8523_registerCallback (uint8_t (*pFunc)(void))
{
  pcf8523_pFunc = pFunc;
}

/******************************************************************************
 *
 * Description:
 *    Initialize the PCF8523 Driver
 *
 *****************************************************************************/
void pcf8523_init (void)
{
  uint8_t cmd[2];
  uint8_t dummy;

  rtcFlag = FALSE;

  //Set PIO0_11 to be GPIO and have pullup enabled
  LPC_IOCON->TDI_PIO0_11 = (1<<0) | (2<<3) | (1<<7);

  //Set PIO0_11 to be an input
  GPIOSetDir( 0, 11, 0 );

  cmd[0] = PCF8523_REG_CONTROL_1; //Set address pointer
  cmd[1] = 0x58;
  //I2C_write_read(i2c-addr,      *pBuf, len, *pBuf2, len2
  I2C_write_read(PCF8523_I2C_ADDR, cmd,   2,   NULL,   0);

  cmd[0] = PCF8523_REG_CONTROL_1; //Set address pointer
#ifdef RTC_CRYSTAL_IS_7PF
  cmd[1] = 0x04; //7pF crystal and enable second IRQ
#else
  cmd[1] = 0x80 | 0x04; //12.5pF crystal and enable second IRQ
#endif
  //I2C_write_read(i2c-addr,      *pBuf, len, *pBuf2, len2
  I2C_write_read(PCF8523_I2C_ADDR, cmd,   2,   NULL,   0);

  cmd[0] = PCF8523_REG_TMR_CLKOUT_CTRL; //Set address pointer
  cmd[1] = 0xB8;  //pulsed 1Hz interrupt signal
//  cmd[1] = 0x38;  //no CLKOUT signal   (cmd[1] = 0x30;  //1Hz CLKOUT signal)
  //I2C_write_read(i2c-addr,      *pBuf, len, *pBuf2, len2
  I2C_write_read(PCF8523_I2C_ADDR, cmd,   2,   NULL,   0);

  //initialize P0.11 as edge sensitive interrupt, active falling edge
  GPIOSetFlexInterrupt( CHANNEL1, PORT0, 11, 0, 0 );
  NVIC_SetPriority(FLEX_INT1_IRQn, 2);
  LPC_SYSCON->STARTERP0 |= (0x1<<1);  //This interrupt can wakeup the mcu from power down mode
  GPIOFlexIntEnable( CHANNEL1, 0 );

  //clear possible interrupts
  dummy = pcf8523_readCTRL2();
  pcf8523_writeCTRL2(0);
}


